# -*- coding: utf-8 -*-
'''
    :codeauthor: Mike Place <mp@saltstack.com>
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import errno
import socket
import logging

import tornado.gen
import tornado.ioloop
import tornado.testing

import salt.config
import salt.exceptions
import salt.transport.ipc
import salt.transport.server
import salt.transport.client
import salt.utils.platform

from salt.ext import six
from salt.ext.six.moves import range

# Import Salt Testing libs
from tests.support.mock import MagicMock
from tests.support.paths import TMP
from tests.support.unit import skipIf

log = logging.getLogger(__name__)


@skipIf(salt.utils.platform.is_windows(), 'Windows does not support Posix IPC')
class BaseIPCReqCase(tornado.testing.AsyncTestCase):
    '''
    Test the req server/client pair
    '''
    def setUp(self):
        super(BaseIPCReqCase, self).setUp()
        #self._start_handlers = dict(self.io_loop._handlers)
        self.socket_path = os.path.join(TMP, 'ipc_test.ipc')

        self.server_channel = salt.transport.ipc.IPCMessageServer(
            self.socket_path,
            io_loop=self.io_loop,
            payload_handler=self._handle_payload,
        )
        self.server_channel.start()

        self.payloads = []

    def tearDown(self):
        super(BaseIPCReqCase, self).tearDown()
        #failures = []
        try:
            self.server_channel.close()
        except socket.error as exc:
            if exc.errno != errno.EBADF:
                # If its not a bad file descriptor error, raise
                raise
        os.unlink(self.socket_path)
        #for k, v in six.iteritems(self.io_loop._handlers):
        #    if self._start_handlers.get(k) != v:
        #        failures.append((k, v))
        #if len(failures) > 0:
        #    raise Exception('FDs still attached to the IOLoop: {0}'.format(failures))
        del self.payloads
        del self.socket_path
        del self.server_channel
        #del self._start_handlers

    @tornado.gen.coroutine
    def _handle_payload(self, payload, reply_func):
        self.payloads.append(payload)
        yield reply_func(payload)
        if isinstance(payload, dict) and payload.get('stop'):
            self.stop()


class IPCMessageClient(BaseIPCReqCase):
    '''
    Test all of the clear msg stuff
    '''

    def _get_channel(self):
        channel = salt.transport.ipc.IPCMessageClient(
            socket_path=self.socket_path,
            io_loop=self.io_loop,
        )
        channel.connect(callback=self.stop)
        self.wait()
        return channel

    def setUp(self):
        super(IPCMessageClient, self).setUp()
        self.channel = self._get_channel()

    def tearDown(self):
        super(IPCMessageClient, self).tearDown()
        try:
            self.channel.close()
        except socket.error as exc:
            if exc.errno != errno.EBADF:
                # If its not a bad file descriptor error, raise
                raise

    def test_basic_send(self):
        msg = {'foo': 'bar', 'stop': True}
        self.channel.send(msg)
        self.wait()
        self.assertEqual(self.payloads[0], msg)

    def test_many_send(self):
        msgs = []
        self.server_channel.stream_handler = MagicMock()

        for i in range(0, 1000):
            msgs.append('test_many_send_{0}'.format(i))

        for i in msgs:
            self.channel.send(i)
        self.channel.send({'stop': True})
        self.wait()
        self.assertEqual(self.payloads[:-1], msgs)

    def test_very_big_message(self):
        long_str = ''.join([six.text_type(num) for num in range(10**5)])
        msg = {'long_str': long_str, 'stop': True}
        self.channel.send(msg)
        self.wait()
        self.assertEqual(msg, self.payloads[0])

    def test_multistream_sends(self):
        local_channel = self._get_channel()

        for c in (self.channel, local_channel):
            c.send('foo')

        self.channel.send({'stop': True})
        self.wait()
        self.assertEqual(self.payloads[:-1], ['foo', 'foo'])

    def test_multistream_errors(self):
        local_channel = self._get_channel()

        for c in (self.channel, local_channel):
            c.send(None)

        for c in (self.channel, local_channel):
            c.send('foo')

        self.channel.send({'stop': True})
        self.wait()
        self.assertEqual(self.payloads[:-1], [None, None, 'foo', 'foo'])



# -*- coding: utf-8 -*-
'''
Tests for the spm repo
'''
# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import shutil

# Import Salt Testing libs
from tests.support.case import SPMCase
from tests.support.helpers import destructiveTest


@destructiveTest
class SPMRepoTest(SPMCase):
    '''
    Validate commands related to spm repo
    '''
    def setUp(self):
        self.config = self._spm_config()
        self._spm_build_files(self.config)

    def test_spm_create_update_repo(self):
        '''
        test spm create_repo
        '''
        self._spm_create_update_repo(self.config)

        self.assertTrue(os.path.exists(self.config['spm_db']))

        l_repo_file = os.path.join(self.config['spm_cache_dir'], 'local_repo.p')
        self.assertTrue(os.path.exists(l_repo_file))

    def tearDown(self):
        shutil.rmtree(self._tmp_spm)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Bo Maryniuk <bo@suse.de>
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import logging
import multiprocessing

# Import Salt Testing libs
from tests.support.unit import TestCase, skipIf
from tests.support.mock import patch, MagicMock, NO_MOCK, NO_MOCK_REASON
from tests.support.mixins import SaltClientTestCaseMixin

# Import Salt libs
import salt.cli.daemons as daemons

log = logging.getLogger(__name__)


class LoggerMock(object):
    '''
    Logger data collector
    '''

    def __init__(self):
        '''
        init
        :return:
        '''
        self.reset()

    def reset(self):
        '''
        Reset values

        :return:
        '''
        self.messages = []

    def info(self, message, *args, **kwargs):
        '''
        Collects the data from the logger of info type.

        :param data:
        :return:
        '''
        self.messages.append({'message': message, 'args': args,
                              'kwargs': kwargs, 'type': 'info'})

    def warning(self, message, *args, **kwargs):
        '''
        Collects the data from the logger of warning type.

        :param data:
        :return:
        '''
        self.messages.append({'message': message, 'args': args,
                              'kwargs': kwargs, 'type': 'warning'})

    def has_message(self, msg, log_type=None):
        '''
        Check if log has message.

        :param data:
        :return:
        '''
        for data in self.messages:
            log_str = data['message'] % data['args']  # pylint: disable=incompatible-py3-code
            if (data['type'] == log_type or not log_type) and log_str.find(msg) > -1:
                return True

        return False


def _master_exec_test(child_pipe):
    def _create_master():
        '''
        Create master instance
        :return:
        '''
        obj = daemons.Master()
        obj.config = {'user': 'dummy', 'hash_type': alg}
        for attr in ['start_log_info', 'prepare', 'shutdown', 'master']:
            setattr(obj, attr, MagicMock())

        return obj

    _logger = LoggerMock()
    ret = True
    try:
        with patch('salt.cli.daemons.check_user', MagicMock(return_value=True)):
            with patch('salt.cli.daemons.log', _logger):
                for alg in ['md5', 'sha1']:
                    _create_master().start()
                    ret = ret and _logger.has_message(
                        'Do not use {alg}'.format(alg=alg),
                        log_type='warning')

                _logger.reset()

                for alg in ['sha224', 'sha256', 'sha384', 'sha512']:
                    _create_master().start()
                    ret = ret and _logger.messages \
                          and not _logger.has_message('Do not use ')
    except Exception:
        log.exception('Exception raised in master daemon unit test')
        ret = False
    child_pipe.send(ret)
    child_pipe.close()


def _minion_exec_test(child_pipe):
    def _create_minion():
        '''
        Create minion instance
        :return:
        '''
        obj = daemons.Minion()
        obj.config = {'user': 'dummy', 'hash_type': alg}
        for attr in ['start_log_info', 'prepare', 'shutdown']:
            setattr(obj, attr, MagicMock())
        setattr(obj, 'minion', MagicMock(restart=False))

        return obj

    ret = True
    try:
        _logger = LoggerMock()
        with patch('salt.cli.daemons.check_user', MagicMock(return_value=True)):
            with patch('salt.cli.daemons.log', _logger):
                for alg in ['md5', 'sha1']:
                    _create_minion().start()
                    ret = ret and _logger.has_message(
                        'Do not use {alg}'.format(alg=alg),
                        log_type='warning')
                _logger.reset()

                for alg in ['sha224', 'sha256', 'sha384', 'sha512']:
                    _create_minion().start()
                    ret = ret and _logger.messages \
                          and not _logger.has_message('Do not use ')
    except Exception:
        log.exception('Exception raised in minion daemon unit test')
        ret = False
    child_pipe.send(ret)
    child_pipe.close()


def _proxy_exec_test(child_pipe):
    def _create_proxy_minion():
        '''
        Create proxy minion instance
        :return:
        '''
        obj = daemons.ProxyMinion()
        obj.config = {'user': 'dummy', 'hash_type': alg}
        for attr in ['minion', 'start_log_info', 'prepare', 'shutdown', 'tune_in']:
            setattr(obj, attr, MagicMock())

        obj.minion.restart = False
        return obj

    ret = True
    try:
        _logger = LoggerMock()
        with patch('salt.cli.daemons.check_user', MagicMock(return_value=True)):
            with patch('salt.cli.daemons.log', _logger):
                for alg in ['md5', 'sha1']:
                    _create_proxy_minion().start()
                    ret = ret and _logger.has_message(
                        'Do not use {alg}'.format(alg=alg),
                        log_type='warning')

                _logger.reset()

                for alg in ['sha224', 'sha256', 'sha384', 'sha512']:
                    _create_proxy_minion().start()
                    ret = ret and _logger.messages \
                          and not _logger.has_message('Do not use ')
    except Exception:
        log.exception('Exception raised in proxy daemon unit test')
        ret = False
    child_pipe.send(ret)
    child_pipe.close()


def _syndic_exec_test(child_pipe):
    def _create_syndic():
        '''
        Create syndic instance
        :return:
        '''
        obj = daemons.Syndic()
        obj.config = {'user': 'dummy', 'hash_type': alg}
        for attr in ['syndic', 'start_log_info', 'prepare', 'shutdown']:
            setattr(obj, attr, MagicMock())

        return obj

    ret = True
    try:
        _logger = LoggerMock()
        with patch('salt.cli.daemons.check_user', MagicMock(return_value=True)):
            with patch('salt.cli.daemons.log', _logger):
                for alg in ['md5', 'sha1']:
                    _create_syndic().start()
                    ret = ret and _logger.has_message(
                        'Do not use {alg}'.format(alg=alg),
                        log_type='warning')

                _logger.reset()

                for alg in ['sha224', 'sha256', 'sha384', 'sha512']:
                    _create_syndic().start()
                    ret = ret and _logger.messages \
                          and not _logger.has_message('Do not use ')
    except Exception:
        log.exception('Exception raised in syndic daemon unit test')
        ret = False
    child_pipe.send(ret)
    child_pipe.close()


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DaemonsStarterTestCase(TestCase, SaltClientTestCaseMixin):
    '''
    Unit test for the daemons starter classes.
    '''

    def _multiproc_exec_test(self, exec_test):
        m_parent, m_child = multiprocessing.Pipe()
        p_ = multiprocessing.Process(target=exec_test, args=(m_child,))
        p_.start()
        self.assertTrue(m_parent.recv())
        p_.join()

    def test_master_daemon_hash_type_verified(self):
        '''
        Verify if Master is verifying hash_type config option.

        :return:
        '''
        self._multiproc_exec_test(_master_exec_test)

    def test_minion_daemon_hash_type_verified(self):
        '''
        Verify if Minion is verifying hash_type config option.

        :return:
        '''
        self._multiproc_exec_test(_minion_exec_test)

    def test_proxy_minion_daemon_hash_type_verified(self):
        '''
        Verify if ProxyMinion is verifying hash_type config option.

        :return:
        '''
        self._multiproc_exec_test(_proxy_exec_test)

    def test_syndic_daemon_hash_type_verified(self):
        '''
        Verify if Syndic is verifying hash_type config option.

        :return:
        '''
        self._multiproc_exec_test(_syndic_exec_test)



# -*- coding: utf-8 -*-
'''
Unit Tests for functions located in salt.utils.doc.py.
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt libs
import salt.utils.doc

# Import Salt Testing libs
from tests.support.unit import TestCase


class DocUtilsTestCase(TestCase):
    '''
    Test case for doc util.
    '''

    def test_parse_docstring(self):
        test_keystone_str = '''Management of Keystone users
                                ============================

                                :depends:   - keystoneclient Python module
                                :configuration: See :py:mod:`salt.modules.keystone` for setup instructions.
'''

        ret = salt.utils.doc.parse_docstring(test_keystone_str)
        expected_dict = {'deps': ['keystoneclient'],
                         'full': 'Management of Keystone users\n                                '
                                 '============================\n\n                                '
                                 ':depends:   - keystoneclient Python module\n                                '
                                 ':configuration: See :py:mod:`salt.modules.keystone` for setup instructions.\n'}
        self.assertDictEqual(ret, expected_dict)



# -*- coding: utf-8 -*-
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

# Import Salt Libs
import salt.states.boto_kinesis as boto_kinesis


@skipIf(NO_MOCK, NO_MOCK_REASON)
class BotoKinesisTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.boto_kinesis
    '''
    # 'present' function tests: 1

    maxDiff = None

    def setup_loader_modules(self):
        return {boto_kinesis: {}}

    def test_stream_present(self):
        '''
        Test to ensure the kinesis stream exists.
        '''
        name = 'new_stream'
        retention_hours = 24
        enhanced_monitoring = ['IteratorAgeMilliseconds']
        different_enhanced_monitoring = ['IncomingBytes']
        num_shards = 1

        ret = {'name': name,
               'result': True,
               'changes': {},
               'comment': ''}

        shards = [{'ShardId': 'shardId-000000000000',
                   'HashKeyRange': {'EndingHashKey': 'big number', 'StartingHashKey': '0'},
                   'SequenceNumberRange': {'StartingSequenceNumber': 'bigger number'}}]
        stream_description = {'HasMoreShards': False,
                              'RetentionPeriodHours': retention_hours,
                              'StreamName': name,
                              'Shards': shards,
                              'StreamARN': "",
                              'EnhancedMonitoring': [{'ShardLevelMetrics': enhanced_monitoring}],
                              'StreamStatus': 'ACTIVE'}

        exists_mock = MagicMock(side_effect=[{'result': True}, {'result': False}, {'result': True}, {'result': False}])
        get_stream_mock = MagicMock(return_value={'result': {'StreamDescription': stream_description}})
        shard_mock = MagicMock(return_value=[0, 0, {'OpenShards': shards}])
        dict_mock = MagicMock(return_value={'result': True})
        mock_bool = MagicMock(return_value=True)
        with patch.dict(boto_kinesis.__salt__,
                        {'boto_kinesis.exists': exists_mock,
                         'boto_kinesis.create_stream': dict_mock,
                         'boto_kinesis.get_stream_when_active': get_stream_mock,
                         'boto_kinesis.get_info_for_reshard': shard_mock,
                         'boto_kinesis.num_shards_matches': mock_bool}):
            # already present, no change required
            comt = ('Kinesis stream {0} already exists,\n'
                    'Kinesis stream {0}: retention hours did not require change, already set at {1},\n'
                    'Kinesis stream {0}: enhanced monitoring did not require change, already set at {2},\n'
                    'Kinesis stream {0}: did not require resharding, remains at {3} shards'
                    .format(name, retention_hours, enhanced_monitoring, num_shards))
            ret.update({'comment': comt})
            self.assertDictEqual(boto_kinesis.present(name, retention_hours, enhanced_monitoring, num_shards), ret)

            with patch.dict(boto_kinesis.__opts__, {'test': True}):
                # not present, test environment (dry run)
                comt = ('Kinesis stream {0} would be created'
                        .format(name))
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(boto_kinesis.present(name, retention_hours, enhanced_monitoring, num_shards), ret)

                # already present, changes required, test environment (dry run)
                comt = ('Kinesis stream {0} already exists,\n'
                        'Kinesis stream {0}: retention hours would be updated to {1},\n'
                        'Kinesis stream {0}: would enable enhanced monitoring for {2},\n'
                        'Kinesis stream {0}: would disable enhanced monitoring for {3},\n'
                        'Kinesis stream {0}: would be resharded from {4} to {5} shards'
                        .format(name, retention_hours+1, different_enhanced_monitoring,
                                enhanced_monitoring, num_shards, num_shards+1))
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(boto_kinesis.present(name, retention_hours+1, different_enhanced_monitoring,
                                                          num_shards+1), ret)

            # not present, create and configure
            changes = {'new': {'name': name,
                               'num_shards': num_shards}}

            with patch.dict(boto_kinesis.__opts__, {'test': False}):
                comt = ('Kinesis stream {0} successfully created,\n'
                        'Kinesis stream {0}: retention hours did not require change, already set at {1},\n'
                        'Kinesis stream {0}: enhanced monitoring did not require change, already set at {2},\n'
                        'Kinesis stream {0}: did not require resharding, remains at {3} shards'
                        .format(name, retention_hours, enhanced_monitoring, num_shards))
                ret.update({'comment': comt, 'result': True,
                            'changes': changes})
                self.assertDictEqual(ret, boto_kinesis.present(name, retention_hours, enhanced_monitoring, num_shards))

    # 'absent' function tests: 1

    def test_absent(self):
        '''
        Test to ensure the Kinesis stream does not exist.
        '''
        name = 'new_stream'

        ret = {'name': name,
               'result': True,
               'changes': {},
               'comment': ''}

        mock = MagicMock(side_effect=[{'result': False}, {'result': True}, {'result': True}])
        mock_bool = MagicMock(return_value={'result': True})
        with patch.dict(boto_kinesis.__salt__,
                        {'boto_kinesis.exists': mock,
                         'boto_kinesis.delete_stream': mock_bool}):
            comt = ('Kinesis stream {0} does not exist'.format(name))
            ret.update({'comment': comt})
            self.assertDictEqual(boto_kinesis.absent(name), ret)

            with patch.dict(boto_kinesis.__opts__, {'test': True}):
                comt = ('Kinesis stream {0} would be deleted'.format(name))
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(boto_kinesis.absent(name), ret)

            changes = {'new': 'Stream {0} deleted'.format(name),
                       'old': 'Stream {0} exists'.format(name)}

            with patch.dict(boto_kinesis.__opts__, {'test': False}):
                comt = ('Deleted stream {0}'.format(name))
                ret.update({'comment': comt, 'result': True,
                            'changes': changes})
                self.assertDictEqual(boto_kinesis.absent(name), ret)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.unit import skipIf

# Import Salt libs
import salt.utils.platform


@skipIf(not salt.utils.platform.is_windows(), 'windows test only')
class WinServermanagerTest(ModuleCase):
    '''
    Test for salt.modules.win_servermanager
    '''
    def test_list_available(self):
        '''
        Test list available features to install
        '''
        cmd = self.run_function('win_servermanager.list_available')
        self.assertIn('DNS', cmd)
        self.assertIn('NetworkController', cmd)
        self.assertIn('RemoteAccess', cmd)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function
import os

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.config
import salt.utils.path
from salt.syspaths import BASE_FILE_ROOTS_DIR
import salt.states.winrepo as winrepo


class MockRunnerClient(object):
    '''
        Mock RunnerClient class
    '''
    def __init__(self):
        pass

    class RunnerClient(object):
        '''
            Mock RunnerClient class
        '''
        def __init__(self, master_config):
            '''
                init method
            '''
            pass

        @staticmethod
        def cmd(arg1, arg2):
            '''
                Mock cmd method
            '''
            return []


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinrepoTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Validate the winrepo state
    '''
    def setup_loader_modules(self):
        patcher = patch('salt.states.winrepo.salt.runner', MockRunnerClient)
        patcher.start()
        self.addCleanup(patcher.stop)
        return {winrepo: {}}

    def test_genrepo(self):
        '''
        Test to refresh the winrepo.p file of the repository
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': False,
               'comment': ''}
        ret.update({'comment': '{0} is missing'.format(
            os.sep.join([BASE_FILE_ROOTS_DIR, 'win', 'repo']))})
        self.assertDictEqual(winrepo.genrepo('salt'), ret)

        mock = MagicMock(return_value={'winrepo_dir': 'salt',
                                       'winrepo_cachefile': 'abc'})
        with patch.object(salt.config, 'master_config', mock):
            mock = MagicMock(return_value=[0, 1, 2, 3, 4, 5, 6, 7, 8])
            with patch.object(os, 'stat', mock):
                mock = MagicMock(return_value=[])
                with patch.object(salt.utils.path, 'os_walk', mock):
                    with patch.dict(winrepo.__opts__, {'test': True}):
                        ret.update({'comment': '', 'result': None})
                        self.assertDictEqual(winrepo.genrepo('salt'), ret)

                    with patch.dict(winrepo.__opts__, {'test': False}):
                        ret.update({'result': True})
                        self.assertDictEqual(winrepo.genrepo('salt'), ret)

                        ret.update({'changes': {'winrepo': []}})
                        self.assertDictEqual(winrepo.genrepo('salt', True),
                                             ret)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing libs
from tests.support.unit import TestCase

# Import Salt libs
import salt.utils.hashutils


class HashutilsTestCase(TestCase):

    hmac_secret = 's00p3r s3kr1t'

    # Use a non-ascii unicode type to confirm that no UnicodeEncodeError is
    # raised on Python 2.
    str = 'спам'
    str_b64encode_result = '0YHQv9Cw0Lw='
    str_encodestring_result = '0YHQv9Cw0Lw=\n'
    str_md5 = 'a035ac08ab2f03556f9b3ee640052e5c'
    str_sha256 = '095291ffa3d361436d4617879e22c1da06c6ab61a3fb081321ec854a27a091ac'
    str_sha512 = '12efd90e507289f1f21e5dcfe2e92cf0bb4904abccb55c3ce9177670c711981501054b32b807c37058675590d1c484bd2b72a4215a2fa397aa4f2b12f298b1f0'
    str_hmac_challenge = b'qz2k0t1aevKEme3JGsNQJX/xpmf+/w3q6qmWDk1ZqbY='

    # 16 bytes of random data
    bytes = b'b\x19\xf6\x86\x0e\x1a\x1cs\x0c\xda&zv\xfc\xa2\xdd'
    bytes_b64encode_result = 'Yhn2hg4aHHMM2iZ6dvyi3Q=='
    bytes_encodestring_result = 'Yhn2hg4aHHMM2iZ6dvyi3Q==\n'
    bytes_md5 = '4d064241724791641dc15930c65f75c8'
    bytes_sha256 = '25711a31c2673a48f3d1f29b25add574697872968e546d266f441de63b17954a'
    bytes_sha512 = '69f1524e602c1599fc374e1e3e2941e6f6949f4f7fe7321304e4e67bb850f3204dd5cbf9c13e231814540c2f5cd370c24ea257771d9fbf311d8f6085bad12b24'
    bytes_hmac_challenge = b'lQibiD9r1Hpo+5JYknaudIKfTx1L5J3U58M9yQOd04c='

    def test_base64_b64encode(self):
        '''
        Ensure that this function converts the value passed to bytes before
        attempting to encode, avoiding a UnicodeEncodeError on Python 2 and a
        TypeError on Python 3.
        '''
        self.assertEqual(
            salt.utils.hashutils.base64_b64encode(self.str),
            self.str_b64encode_result
        )
        self.assertEqual(
            salt.utils.hashutils.base64_b64encode(self.bytes),
            self.bytes_b64encode_result
        )

    def test_base64_b64decode(self):
        '''
        Ensure that this function converts the value passed to a unicode type
        (if possible) on Python 2, and a str type (if possible) on Python 3.
        '''
        self.assertEqual(
            salt.utils.hashutils.base64_b64decode(self.str_b64encode_result),
            self.str
        )
        self.assertEqual(
            salt.utils.hashutils.base64_b64decode(self.bytes_b64encode_result),
            self.bytes
        )

    def test_base64_encodestring(self):
        '''
        Ensure that this function converts the value passed to bytes before
        attempting to encode, avoiding a UnicodeEncodeError on Python 2 and a
        TypeError on Python 3.
        '''
        self.assertEqual(
            salt.utils.hashutils.base64_encodestring(self.str),
            self.str_encodestring_result
        )
        self.assertEqual(
            salt.utils.hashutils.base64_encodestring(self.bytes),
            self.bytes_encodestring_result
        )

    def test_base64_decodestring(self):
        '''
        Ensure that this function converts the value passed to a unicode type
        (if possible) on Python 2, and a str type (if possible) on Python 3.
        '''
        self.assertEqual(
            salt.utils.hashutils.base64_decodestring(self.str_encodestring_result),
            self.str
        )
        self.assertEqual(
            salt.utils.hashutils.base64_decodestring(self.bytes_encodestring_result),
            self.bytes
        )

    def test_md5_digest(self):
        '''
        Ensure that this function converts the value passed to bytes before
        attempting to encode, avoiding a UnicodeEncodeError on Python 2 and a
        TypeError on Python 3.
        '''
        self.assertEqual(
            salt.utils.hashutils.md5_digest(self.str),
            self.str_md5
        )
        self.assertEqual(
            salt.utils.hashutils.md5_digest(self.bytes),
            self.bytes_md5
        )

    def test_sha256_digest(self):
        '''
        Ensure that this function converts the value passed to bytes before
        attempting to encode, avoiding a UnicodeEncodeError on Python 2 and a
        TypeError on Python 3.
        '''
        self.assertEqual(
            salt.utils.hashutils.sha256_digest(self.str),
            self.str_sha256
        )
        self.assertEqual(
            salt.utils.hashutils.sha256_digest(self.bytes),
            self.bytes_sha256
        )

    def test_sha512_digest(self):
        '''
        Ensure that this function converts the value passed to bytes before
        attempting to encode, avoiding a UnicodeEncodeError on Python 2 and a
        TypeError on Python 3.
        '''
        self.assertEqual(
            salt.utils.hashutils.sha512_digest(self.str),
            self.str_sha512
        )
        self.assertEqual(
            salt.utils.hashutils.sha512_digest(self.bytes),
            self.bytes_sha512
        )

    def test_hmac_signature(self):
        '''
        Ensure that this function converts the value passed to bytes before
        attempting to validate the hmac challenge, avoiding a
        UnicodeEncodeError on Python 2 and a TypeError on Python 3.
        '''
        self.assertTrue(
            salt.utils.hashutils.hmac_signature(
                self.str,
                self.hmac_secret,
                self.str_hmac_challenge
            )
        )
        self.assertTrue(
            salt.utils.hashutils.hmac_signature(
                self.bytes,
                self.hmac_secret,
                self.bytes_hmac_challenge
            )
        )

    def test_get_hash_exception(self):
        self.assertRaises(
            ValueError,
            salt.utils.hashutils.get_hash,
            '/tmp/foo/',
            form='INVALID')



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.djangomod as djangomod


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DjangomodTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.djangomod
    '''
    def setup_loader_modules(self):
        patcher = patch('salt.utils.path.which', lambda exe: exe)
        patcher.start()
        self.addCleanup(patcher.stop)
        return {djangomod: {'_get_django_admin': MagicMock(return_value=True)}}

    # 'command' function tests: 1

    def test_command(self):
        '''
        Test if it runs arbitrary django management command
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(djangomod.__salt__, {'cmd.run': mock}):
            self.assertTrue(djangomod.command('DJANGO_SETTINGS_MODULE',
                                              'validate'))

    # 'syncdb' function tests: 1

    def test_syncdb(self):
        '''
        Test if it runs the Django-Admin syncdb command
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(djangomod.__salt__, {'cmd.run': mock}):
            self.assertTrue(djangomod.syncdb('DJANGO_SETTINGS_MODULE'))

    # 'createsuperuser' function tests: 1

    def test_createsuperuser(self):
        '''
        Test if it create a super user for the database.
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(djangomod.__salt__, {'cmd.run': mock}):
            self.assertTrue(djangomod.createsuperuser('DJANGO_SETTINGS_MODULE',
                                                      'SALT',
                                                      'salt@slatstack.com'))

    # 'loaddata' function tests: 1

    def test_loaddata(self):
        '''
        Test if it loads fixture data
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(djangomod.__salt__, {'cmd.run': mock}):
            self.assertTrue(djangomod.loaddata('DJANGO_SETTINGS_MODULE',
                                               'mydata'))

    # 'collectstatic' function tests: 1

    def test_collectstatic(self):
        '''
        Test if it collect static files from each of your applications
        into a single location
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(djangomod.__salt__, {'cmd.run': mock}):
            self.assertTrue(djangomod.collectstatic('DJANGO_SETTINGS_MODULE'))


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DjangomodCliCommandTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.djangomod
    '''
    def setup_loader_modules(self):
        patcher = patch('salt.utils.path.which', lambda exe: exe)
        patcher.start()
        self.addCleanup(patcher.stop)
        return {djangomod: {}}

    def test_django_admin_cli_command(self):
        mock = MagicMock()
        with patch.dict(djangomod.__salt__,
                        {'cmd.run': mock}):
            djangomod.command('settings.py', 'runserver')
            mock.assert_called_once_with(
                'django-admin.py runserver --settings=settings.py',
                python_shell=False,
                env=None
            )

    def test_django_admin_cli_command_with_args(self):
        mock = MagicMock()
        with patch.dict(djangomod.__salt__,
                        {'cmd.run': mock}):
            djangomod.command(
                'settings.py',
                'runserver',
                None,
                None,
                None,
                'noinput',
                'somethingelse'
            )
            mock.assert_called_once_with(
                'django-admin.py runserver --settings=settings.py '
                '--noinput --somethingelse',
                python_shell=False,
                env=None
            )

    def test_django_admin_cli_command_with_kwargs(self):
        mock = MagicMock()
        with patch.dict(djangomod.__salt__,
                        {'cmd.run': mock}):
            djangomod.command(
                'settings.py',
                'runserver',
                None,
                None,
                database='something'
            )
            mock.assert_called_once_with(
                'django-admin.py runserver --settings=settings.py '
                '--database=something',
                python_shell=False,
                env=None
            )

    def test_django_admin_cli_command_with_kwargs_ignore_dunder(self):
        mock = MagicMock()
        with patch.dict(djangomod.__salt__,
                        {'cmd.run': mock}):
            djangomod.command(
                'settings.py', 'runserver', None, None, __ignore='something'
            )
            mock.assert_called_once_with(
                'django-admin.py runserver --settings=settings.py',
                python_shell=False,
                env=None
            )

    def test_django_admin_cli_syncdb(self):
        mock = MagicMock()
        with patch.dict(djangomod.__salt__,
                        {'cmd.run': mock}):
            djangomod.syncdb('settings.py')
            mock.assert_called_once_with(
                'django-admin.py syncdb --settings=settings.py --noinput',
                python_shell=False,
                env=None
            )

    def test_django_admin_cli_syncdb_migrate(self):
        mock = MagicMock()
        with patch.dict(djangomod.__salt__,
                        {'cmd.run': mock}):
            djangomod.syncdb('settings.py', migrate=True)
            mock.assert_called_once_with(
                'django-admin.py syncdb --settings=settings.py --migrate '
                '--noinput',
                python_shell=False,
                env=None
            )

    def test_django_admin_cli_createsuperuser(self):
        mock = MagicMock()
        with patch.dict(djangomod.__salt__,
                        {'cmd.run': mock}):
            djangomod.createsuperuser(
                'settings.py', 'testuser', 'user@example.com'
            )
            self.assertEqual(mock.call_count, 1)
            args, kwargs = mock.call_args
            # cmdline arguments are extracted from a kwargs dict so order isn't guaranteed.
            self.assertEqual(len(args), 1)
            self.assertTrue(args[0].startswith('django-admin.py createsuperuser --'))
            self.assertEqual(set(args[0].split()),
                             set('django-admin.py createsuperuser --settings=settings.py --noinput '
                                                   '--username=testuser --email=user@example.com'.split()))
            self.assertDictEqual(kwargs, {'python_shell': False, 'env': None})

    def no_test_loaddata(self):
        mock = MagicMock()
        with patch.dict(djangomod.__salt__,
                        {'cmd.run': mock}):
            djangomod.loaddata('settings.py', 'app1,app2')
            mock.assert_called_once_with(
                'django-admin.py loaddata --settings=settings.py app1 app2',
            )

    def test_django_admin_cli_collectstatic(self):
        mock = MagicMock()
        with patch.dict(djangomod.__salt__,
                        {'cmd.run': mock}):
            djangomod.collectstatic(
                'settings.py', None, True, 'something', True, True, True, True
            )
            mock.assert_called_once_with(
                'django-admin.py collectstatic --settings=settings.py '
                '--noinput --no-post-process --dry-run --clear --link '
                '--no-default-ignore --ignore=something',
                python_shell=False,
                env=None
            )



# -*- coding: utf-8 -*-
'''
unittests for json outputter
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase
from tests.support.mock import patch

# Import Salt Libs
import salt.output.json_out as json_out
import salt.utils.stringutils
from salt.ext import six


class JsonTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.output.json_out
    '''
    def setup_loader_modules(self):
        return {json_out: {}}

    def setUp(self):
        self.data = {'test': 'two', 'example': 'one'}
        self.addCleanup(delattr, self, 'data')

    def test_default_output(self):
        ret = json_out.output(self.data)
        self.assertIn('"test": "two"', ret)
        self.assertIn('"example": "one"', ret)

    def test_pretty_output(self):
        with patch.dict(json_out.__opts__, {'output_indent': 'pretty'}):
            ret = json_out.output(self.data)
            self.assertIn('"test": "two"', ret)
            self.assertIn('"example": "one"', ret)

    def test_indent_output(self):
        with patch.dict(json_out.__opts__, {'output_indent': 2}):
            ret = json_out.output(self.data)
            self.assertIn('"test": "two"', ret)
            self.assertIn('"example": "one"', ret)

    def test_negative_zero_output(self):
        with patch.dict(json_out.__opts__, {'output_indent': 0}):
            ret = json_out.output(self.data)
            self.assertIn('"test": "two"', ret)
            self.assertIn('"example": "one"', ret)

    def test_negative_int_output(self):
        with patch.dict(json_out.__opts__, {'output_indent': -1}):
            ret = json_out.output(self.data)
            self.assertIn('"test": "two"', ret)
            self.assertIn('"example": "one"', ret)

    def test_unicode_output(self):
        with patch.dict(json_out.__opts__, {'output_indent': 'pretty'}):
            decoded = {'test': 'Д', 'example': 'one'}
            encoded = {'test': salt.utils.stringutils.to_str('Д'), 'example': 'one'}
            # json.dumps on Python 2 adds a space before a newline while in the
            # process of dumping a dictionary.
            if six.PY2:
                expected = salt.utils.stringutils.to_str('{\n    "example": "one", \n    "test": "Д"\n}')
            else:
                expected = '{\n    "example": "one",\n    "test": "Д"\n}'
            self.assertEqual(json_out.output(decoded), expected)
            self.assertEqual(json_out.output(encoded), expected)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Mike Place <mp@saltstack.com>
'''

# Import python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch, call

# Import Salt libs
import salt.modules.cron as cron
from salt.ext.six.moves import builtins, StringIO

STUB_USER = 'root'
STUB_PATH = '/tmp'

STUB_CRON_TIMESTAMP = {'minute': '1',
                       'hour': '2',
                       'daymonth': '3',
                       'month': '4',
                       'dayweek': '5'}

STUB_SIMPLE_RAW_CRON = '5 0 * * * /tmp/no_script.sh'
STUB_SIMPLE_CRON_DICT = {'pre': ['5 0 * * * /tmp/no_script.sh'], 'crons': [], 'env': [], 'special': []}
STUB_CRON_SPACES = """
# Lines below here are managed by Salt, do not edit
TEST_VAR="a string with plenty of spaces"
# SALT_CRON_IDENTIFIER:echo "must  be  double  spaced"
11 * * * * echo "must  be  double  spaced"
"""

L = '# Lines below here are managed by Salt, do not edit\n'

CRONTAB = StringIO()


def get_crontab(*args, **kw):
    return CRONTAB.getvalue()


def set_crontab(val):
    CRONTAB.seek(0)
    CRONTAB.truncate(0)
    CRONTAB.write(val)


def write_crontab(*args, **kw):
    set_crontab('\n'.join(
        [a.strip() for a in args[1]]))
    return MagicMock()


@skipIf(NO_MOCK, NO_MOCK_REASON)
class CronTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {cron: {}}

    def test__need_changes_new(self):
        '''
        New behavior, identifier will get track of the managed lines!
        '''
        with patch('salt.modules.cron.raw_cron',
                   new=MagicMock(side_effect=get_crontab)), \
                           patch('salt.modules.cron._write_cron_lines',
                                 new=MagicMock(side_effect=write_crontab)):
            # when there are no identifiers,
            # we do not touch it
            set_crontab(
                L + '# SALT_CRON_IDENTIFIER:booh\n'
                '* * * * * ls\n')
            cron.set_job(
                user='root',
                minute='*',
                hour='*',
                daymonth='*',
                month='*',
                dayweek='*',
                cmd='ls',
                comment=None,
                identifier=None,
            )
            c1 = get_crontab()
            set_crontab(L + '* * * * * ls\n')
            self.assertEqual(
                c1,
                '# Lines below here are managed by Salt, do not edit\n'
                '# SALT_CRON_IDENTIFIER:booh\n'
                '* * * * * ls\n'
                '* * * * * ls'
            )
            # whenever we have an identifier, hourray even without comment
            # we can match and edit the crontab in place
            # without cluttering the crontab with new cmds
            set_crontab(
                L + '# SALT_CRON_IDENTIFIER:bar\n'
                '* * * * * ls\n')
            cron.set_job(
                user='root',
                minute='*',
                hour='*',
                daymonth='*',
                month='*',
                dayweek='*',
                cmd='ls',
                comment=None,
                identifier='bar',
            )
            c5 = get_crontab()
            set_crontab(L + '* * * * * ls\n')
            self.assertEqual(
                c5,
                '# Lines below here are managed by Salt, do not edit\n'
                '# SALT_CRON_IDENTIFIER:bar\n'
                '* * * * * ls\n'
            )
            # we can even change the other parameters as well
            # thx to the id
            set_crontab(
                L + '# SALT_CRON_IDENTIFIER:bar\n* * * * * ls\n')
            cron.set_job(
                user='root',
                minute='1',
                hour='2',
                daymonth='3',
                month='4',
                dayweek='5',
                cmd='foo',
                comment='moo',
                identifier='bar',
            )
            c6 = get_crontab()
            self.assertEqual(
                c6,
                '# Lines below here are managed by Salt, do not edit\n'
                '# moo SALT_CRON_IDENTIFIER:bar\n'
                '1 2 3 4 5 foo'
            )

    def test__unicode_match(self):
        with patch.object(builtins, '__salt_system_encoding__', 'utf-8'):
            self.assertTrue(cron._cron_matched({'identifier': '1'}, 'foo', 1))
            self.assertTrue(cron._cron_matched({'identifier': 'é'}, 'foo', 'é'))
            self.assertTrue(cron._cron_matched({'identifier': u'é'}, 'foo', 'é'))
            self.assertTrue(cron._cron_matched({'identifier': 'é'}, 'foo', u'é'))
            self.assertTrue(cron._cron_matched({'identifier': u'é'}, 'foo', u'é'))

    def test__need_changes_old(self):
        '''
        old behavior; ID has no special action
        - If an id is found, it will be added as a new crontab
          even if there is a cmd that looks like this one
        - no comment, delete the cmd and readd it
        - comment: idem
        '''
        with patch('salt.modules.cron.raw_cron',
                   new=MagicMock(side_effect=get_crontab)), \
                        patch('salt.modules.cron._write_cron_lines',
                              new=MagicMock(side_effect=write_crontab)):
            set_crontab(L + '* * * * * ls\n\n')
            cron.set_job(
                user='root',
                minute='*',
                hour='*',
                daymonth='*',
                month='*',
                dayweek='*',
                cmd='ls',
                comment=None,
                identifier=cron.SALT_CRON_NO_IDENTIFIER,
            )
            c1 = get_crontab()
            set_crontab(L + '* * * * * ls\n')
            self.assertEqual(
                c1,
                '# Lines below here are managed by Salt, do not edit\n'
                '* * * * * ls\n'
                '\n'
            )
            cron.set_job(
                user='root',
                minute='*',
                hour='*',
                daymonth='*',
                month='*',
                dayweek='*',
                cmd='ls',
                comment='foo',
                identifier=cron.SALT_CRON_NO_IDENTIFIER,
            )
            c2 = get_crontab()
            self.assertEqual(
                c2,
                '# Lines below here are managed by Salt, do not edit\n'
                '# foo\n* * * * * ls'
            )
            set_crontab(L + '* * * * * ls\n')
            cron.set_job(
                user='root',
                minute='*',
                hour='*',
                daymonth='*',
                month='*',
                dayweek='*',
                cmd='lsa',
                comment='foo',
                identifier='bar',
            )
            c3 = get_crontab()
            self.assertEqual(
                c3,
                '# Lines below here are managed by Salt, do not edit\n'
                '* * * * * ls\n'
                '# foo SALT_CRON_IDENTIFIER:bar\n'
                '* * * * * lsa'
            )
            set_crontab(L + '* * * * * ls\n')
            cron.set_job(
                user='root',
                minute='*',
                hour='*',
                daymonth='*',
                month='*',
                dayweek='*',
                cmd='foo',
                comment='foo',
                identifier='bar',
            )
            c4 = get_crontab()
            self.assertEqual(
                c4,
                '# Lines below here are managed by Salt, do not edit\n'
                '* * * * * ls\n'
                '# foo SALT_CRON_IDENTIFIER:bar\n'
                '* * * * * foo'
            )
            set_crontab(L + '* * * * * ls\n')
            cron.set_job(
                user='root',
                minute='*',
                hour='*',
                daymonth='*',
                month='*',
                dayweek='*',
                cmd='ls',
                comment='foo',
                identifier='bbar',
            )
            c4 = get_crontab()
            self.assertEqual(
                c4,
                '# Lines below here are managed by Salt, do not edit\n'
                '# foo SALT_CRON_IDENTIFIER:bbar\n'
                '* * * * * ls'
            )

    def test__issue10959(self):
        '''
        handle multi old style crontabs
        https://github.com/saltstack/salt/issues/10959
        '''
        with patch('salt.modules.cron.raw_cron',
                   new=MagicMock(side_effect=get_crontab)), \
                        patch('salt.modules.cron._write_cron_lines',
                              new=MagicMock(side_effect=write_crontab)):
            set_crontab(
                '# Lines below here are managed by Salt, do not edit\n'
                '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls\n'
                '# uuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * too\n'
                '# uuuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * zoo\n'
                '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * yoo\n'
                '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * xoo\n'
                # as managed per salt, the last lines will be merged together !
                '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * samecmd\n'
                '* * * * * samecmd\n'
                '* * * * * otheridcmd\n'
                '* * * * * otheridcmd\n'
                '# SALT_CRON_IDENTIFIER:NO ID SET\n0 * * * * samecmd1\n'
                '1 * * * * samecmd1\n'
                '0 * * * * otheridcmd1\n'
                '1 * * * * otheridcmd1\n'
                # special case here, none id managed line with same command
                # as a later id managed line will become managed
                '# SALT_CRON_IDENTIFIER:1\n0 * * * * otheridcmd1\n'
                '# SALT_CRON_IDENTIFIER:2\n0 * * * * otheridcmd1\n'
            )
            crons1 = cron.list_tab('root')
            # the filtering is done on save, we reflect in listing
            # the same that we have in a file, no matter what we
            # have
            self.assertEqual(crons1, {
                'crons': [
                    {'cmd': 'ls', 'comment': 'uoo', 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': 'NO ID SET',
                     'minute': '*', 'month': '*', 'commented': False},
                    {'cmd': 'too', 'comment': 'uuoo', 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': 'NO ID SET',
                     'minute': '*', 'month': '*', 'commented': False},
                    {'cmd': 'zoo', 'comment': 'uuuoo', 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': 'NO ID SET',
                     'minute': '*', 'month': '*', 'commented': False},
                    {'cmd': 'yoo', 'comment': '', 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': 'NO ID SET',
                     'minute': '*', 'month': '*', 'commented': False},
                    {'cmd': 'xoo', 'comment': '', 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': 'NO ID SET',
                     'minute': '*', 'month': '*', 'commented': False},
                    {'cmd': 'samecmd', 'comment': '', 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': 'NO ID SET',
                     'minute': '*', 'month': '*', 'commented': False},
                    {'cmd': 'samecmd', 'comment': None, 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': None,
                     'minute': '*', 'month': '*', 'commented': False},
                    {'cmd': 'otheridcmd', 'comment': None, 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': None,
                     'minute': '*', 'month': '*', 'commented': False},
                    {'cmd': 'otheridcmd', 'comment': None, 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': None,
                     'minute': '*', 'month': '*', 'commented': False},
                    {'cmd': 'samecmd1', 'comment': '', 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': 'NO ID SET',
                     'minute': '0', 'month': '*', 'commented': False},
                    {'cmd': 'samecmd1', 'comment': None, 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': None,
                     'minute': '1', 'month': '*', 'commented': False},
                    {'cmd': 'otheridcmd1', 'comment': None, 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': None,
                     'minute': '0', 'month': '*', 'commented': False},
                    {'cmd': 'otheridcmd1', 'comment': None, 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': None,
                     'minute': '1', 'month': '*', 'commented': False},
                    {'cmd': 'otheridcmd1', 'comment': '', 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': '1',
                     'minute': '0', 'month': '*', 'commented': False},
                    {'cmd': 'otheridcmd1',
                     'comment': '', 'daymonth': '*', 'dayweek': '*',
                     'hour': '*', 'identifier': '2', 'minute': '0',
                     'month': '*', 'commented': False}
                ],
                'env': [],
                'pre': [],
                'special': []})
            # so yood so far, no problem for now, trying to save the
            # multilines without id crons now
            inc_tests = [
                ('# Lines below here are managed by Salt, do not edit\n'
                 '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls'),
                #
                ('# Lines below here are managed by Salt, do not edit\n'
                 '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls\n'
                 '# uuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * too'),
                #
                ('# Lines below here are managed by Salt, do not edit\n'
                 '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls\n'
                 '# uuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * too\n'
                 '# uuuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * zoo'),
                #
                ('# Lines below here are managed by Salt, do not edit\n'
                 '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls\n'
                 '# uuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * too\n'
                 '# uuuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * zoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * yoo'),
                #
                ('# Lines below here are managed by Salt, do not edit\n'
                 '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls\n'
                 '# uuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * too\n'
                 '# uuuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * zoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * yoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * xoo'),
                #
                ('# Lines below here are managed by Salt, do not edit\n'
                 '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls\n'
                 '# uuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * too\n'
                 '# uuuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * zoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * yoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * xoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * samecmd'),
                #
                ('# Lines below here are managed by Salt, do not edit\n'
                 '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls\n'
                 '# uuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * too\n'
                 '# uuuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * zoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * yoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * xoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * samecmd'),
                #
                ('# Lines below here are managed by Salt, do not edit\n'
                 '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls\n'
                 '# uuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * too\n'
                 '# uuuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * zoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * yoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * xoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * samecmd\n'
                 '* * * * * otheridcmd'),
                #
                ('# Lines below here are managed by Salt, do not edit\n'
                 '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls\n'
                 '# uuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * too\n'
                 '# uuuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * zoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * yoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * xoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * samecmd\n'
                 '* * * * * otheridcmd'),
                #
                ('# Lines below here are managed by Salt, do not edit\n'
                 '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls\n'
                 '# uuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * too\n'
                 '# uuuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * zoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * yoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * xoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * samecmd\n'
                 '* * * * * otheridcmd\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n'
                 '0 * * * * samecmd1'),
                #
                ('# Lines below here are managed by Salt, do not edit\n'
                 '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls\n'
                 '# uuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * too\n'
                 '# uuuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * zoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * yoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * xoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * samecmd\n'
                 '* * * * * otheridcmd\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n1 * * * * samecmd1'),
                #
                ('# Lines below here are managed by Salt, do not edit\n'
                 '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls\n'
                 '# uuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * too\n'
                 '# uuuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * zoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * yoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * xoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * samecmd\n'
                 '* * * * * otheridcmd\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n1 * * * * samecmd1\n'
                 '0 * * * * otheridcmd1'),
                #
                ('# Lines below here are managed by Salt, do not edit\n'
                 '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls\n'
                 '# uuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * too\n'
                 '# uuuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * zoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * yoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * xoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * samecmd\n'
                 '* * * * * otheridcmd\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n1 * * * * samecmd1\n'
                 '1 * * * * otheridcmd1'),
                #
                ('# Lines below here are managed by Salt, do not edit\n'
                 '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls\n'
                 '# uuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * too\n'
                 '# uuuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * zoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * yoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * xoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * samecmd\n'
                 '* * * * * otheridcmd\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n1 * * * * samecmd1\n'
                 '# SALT_CRON_IDENTIFIER:1\n0 * * * * otheridcmd1'),
                #
                ('# Lines below here are managed by Salt, do not edit\n'
                 '# uoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * ls\n'
                 '# uuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * too\n'
                 '# uuuoo SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * zoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * yoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * xoo\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n* * * * * samecmd\n'
                 '* * * * * otheridcmd\n'
                 '# SALT_CRON_IDENTIFIER:NO ID SET\n1 * * * * samecmd1\n'
                 '# SALT_CRON_IDENTIFIER:1\n0 * * * * otheridcmd1\n'
                 '# SALT_CRON_IDENTIFIER:2\n0 * * * * otheridcmd1')
            ]
            set_crontab('')
            for idx, cr in enumerate(crons1['crons']):
                cron.set_job('root', **cr)
                self.assertEqual(
                    get_crontab(),
                    inc_tests[idx], (
                        "idx {0}\n'{1}'\n != \n'{2}'\n\n\n"
                        "\'{1}\' != \'{2}\'"
                    ).format(
                        idx, get_crontab(), inc_tests[idx]))

    def test_list_tab_commented_cron_jobs(self):
        '''
        handle commented cron jobs
        https://github.com/saltstack/salt/issues/29082
        '''
        with patch('salt.modules.cron.raw_cron', MagicMock(side_effect=get_crontab)), \
                patch('salt.modules.cron._write_cron_lines',
                      MagicMock(side_effect=write_crontab)):
            set_crontab(
                '# An unmanaged commented cron job\n'
                '#0 * * * * /bin/true\n'
                '# Lines below here are managed by Salt, do not edit\n'
                '# cron_1 SALT_CRON_IDENTIFIER:cron_1\n#DISABLED#0 * * * * my_cmd_1\n'
                '# cron_2 SALT_CRON_IDENTIFIER:cron_2\n#DISABLED#* * * * * my_cmd_2\n'
                '# cron_3 SALT_CRON_IDENTIFIER:cron_3\n'
                '#DISABLED#but it is a comment line'
                '#DISABLED#0 * * * * my_cmd_3\n'
                '# cron_4 SALT_CRON_IDENTIFIER:cron_4\n0 * * * * my_cmd_4\n'
            )
            crons1 = cron.list_tab('root')
            self.assertEqual(crons1, {
                'crons': [
                    {'cmd': 'my_cmd_1', 'comment': 'cron_1', 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': 'cron_1',
                     'minute': '0', 'month': '*', 'commented': True},
                    {'cmd': 'my_cmd_2', 'comment': 'cron_2', 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': 'cron_2',
                     'minute': '*', 'month': '*', 'commented': True},
                    {'cmd': 'line#DISABLED#0 * * * * my_cmd_3',
                     'comment': 'cron_3', 'daymonth': 'is',
                     'dayweek': 'comment', 'hour': 'it', 'identifier': 'cron_3',
                     'minute': 'but', 'month': 'a', 'commented': True},
                    {'cmd': 'my_cmd_4', 'comment': 'cron_4', 'daymonth': '*',
                     'dayweek': '*', 'hour': '*', 'identifier': 'cron_4',
                     'minute': '0', 'month': '*', 'commented': False},
                ],
                'env': [],
                'pre': ['# An unmanaged commented cron job', '#0 * * * * /bin/true'],
                'special': []})

    def test_cron_extra_spaces(self):
        '''
        Issue #38449
        '''
        with patch.dict(cron.__grains__, {'os': None}), \
                patch('salt.modules.cron.raw_cron',
                      MagicMock(return_value=STUB_CRON_SPACES)):
            ret = cron.list_tab('root')
            eret = {'crons': [{'cmd': 'echo "must  be  double  spaced"',
                               'comment': '',
                               'commented': False,
                               'daymonth': '*',
                               'dayweek': '*',
                               'hour': '*',
                               'identifier': 'echo "must  be  double  spaced"',
                               'minute': '11',
                               'month': '*'}],
                    'env': [{'name': 'TEST_VAR', 'value': '"a string with plenty of spaces"'}],
                    'pre': [''],
                    'special': []}
            self.assertEqual(eret, ret)

    def test__load_tab(self):
        with patch.dict(cron.__grains__, {'os_family': 'Solaris'}), \
                patch('salt.modules.cron.raw_cron',
                      new=MagicMock(side_effect=[
                          (L + '\n'),
                          (L + '* * * * * ls\nn'),
                          (L + '# commented\n'
                           '#DISABLED#* * * * * ls\n'),
                          (L + '# foo\n'
                           '* * * * * ls\n'),
                          (L + '# foo {0}:blah\n'.format(
                              cron.SALT_CRON_IDENTIFIER) +
                              '* * * * * ls\n')])):
            crons1 = cron.list_tab('root')
            crons2 = cron.list_tab('root')
            crons3 = cron.list_tab('root')
            crons4 = cron.list_tab('root')
            crons5 = cron.list_tab('root')
            self.assertEqual(
                crons1,
                {'pre': [], 'crons': [], 'env': [], 'special': []})
            self.assertEqual(
                crons2['crons'][0],
                {'comment': None,
                 'commented': False,
                 'dayweek': '*',
                 'hour': '*',
                 'identifier': None,
                 'cmd': 'ls',
                 'daymonth': '*',
                 'minute': '*',
                 'month': '*'})
            self.assertEqual(
                crons3['crons'][0],
                {'comment': 'commented',
                 'commented': True,
                 'dayweek': '*',
                 'hour': '*',
                 'identifier': None,
                 'cmd': 'ls',
                 'daymonth': '*',
                 'minute': '*',
                 'month': '*'})
            self.assertEqual(
                crons4['crons'][0],
                {'comment': 'foo',
                 'commented': False,
                 'dayweek': '*',
                 'hour': '*',
                 'identifier': None,
                 'cmd': 'ls',
                 'daymonth': '*',
                 'minute': '*',
                 'month': '*'})
            self.assertEqual(
                crons5['crons'][0],
                {'comment': 'foo',
                 'commented': False,
                 'dayweek': '*',
                 'hour': '*',
                 'identifier': 'blah',
                 'cmd': 'ls',
                 'daymonth': '*',
                 'minute': '*',
                 'month': '*'})

    def test_write_cron_file_root_rh(self):
        '''
        Assert that write_cron_file() is called with the correct cron command and user: RedHat
          - If instance running uid matches crontab user uid, runas STUB_USER without -u flag.
        '''
        with patch.dict(cron.__grains__, {'os_family': 'RedHat'}), \
                patch.dict(cron.__salt__, {'cmd.retcode': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          new=MagicMock(return_value=True)):
            cron.write_cron_file(STUB_USER, STUB_PATH)
            cron.__salt__['cmd.retcode'].assert_called_with("crontab /tmp",
                                                            runas=STUB_USER,
                                                            python_shell=False)

    def test_write_cron_file_foo_rh(self):
        '''
        Assert that write_cron_file() is called with the correct cron command and user: RedHat
          - If instance running with uid that doesn't match crontab user uid, run with -u flag
        '''
        with patch.dict(cron.__grains__, {'os_family': 'RedHat'}), \
                patch.dict(cron.__salt__, {'cmd.retcode': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=False)):
            cron.write_cron_file('foo', STUB_PATH)
            cron.__salt__['cmd.retcode'].assert_called_with("crontab -u foo /tmp",
                                                            python_shell=False)

    def test_write_cron_file_root_sol(self):
        '''
        Assert that write_cron_file() is called with the correct cron command and user: Solaris
          - Solaris should always run without a -u flag
        '''
        with patch.dict(cron.__grains__, {'os_family': 'RedHat'}), \
                patch.dict(cron.__salt__, {'cmd.retcode': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=True)):
            cron.write_cron_file(STUB_USER, STUB_PATH)
            cron.__salt__['cmd.retcode'].assert_called_with("crontab /tmp",
                                                            runas=STUB_USER,
                                                            python_shell=False)

    def test_write_cron_file_foo_sol(self):
        '''
        Assert that write_cron_file() is called with the correct cron command and user: Solaris
          - Solaris should always run without a -u flag
        '''
        with patch.dict(cron.__grains__, {'os_family': 'Solaris'}), \
                patch.dict(cron.__salt__, {'cmd.retcode': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=False)):
            cron.write_cron_file('foo', STUB_PATH)
            cron.__salt__['cmd.retcode'].assert_called_with("crontab /tmp",
                                                            runas='foo',
                                                            python_shell=False)

    def test_write_cron_file_root_aix(self):
        '''
        Assert that write_cron_file() is called with the correct cron command and user: AIX
          - AIX should always run without a -u flag
        '''
        with patch.dict(cron.__grains__, {'os_family': 'AIX'}), \
                patch.dict(cron.__salt__, {'cmd.retcode': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=True)):
            cron.write_cron_file(STUB_USER, STUB_PATH)
            cron.__salt__['cmd.retcode'].assert_called_with("crontab /tmp",
                                                            runas=STUB_USER,
                                                            python_shell=False)

    def test_write_cron_file_foo_aix(self):
        '''
        Assert that write_cron_file() is called with the correct cron command and user: AIX
          - AIX should always run without a -u flag
        '''
        with patch.dict(cron.__grains__, {'os_family': 'AIX'}), \
                patch.dict(cron.__salt__, {'cmd.retcode': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=False)):
            cron.write_cron_file('foo', STUB_PATH)
            cron.__salt__['cmd.retcode'].assert_called_with("crontab /tmp",
                                                            runas='foo',
                                                            python_shell=False)

    def test_write_cr_file_v_root_rh(self):
        '''
        Assert that write_cron_file_verbose() is called with the correct cron command and user: RedHat
          - If instance running uid matches crontab user uid, runas STUB_USER without -u flag.
        '''
        with patch.dict(cron.__grains__, {'os_family': 'Redhat'}), \
                patch.dict(cron.__salt__, {'cmd.run_all': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=True)):
            cron.write_cron_file_verbose(STUB_USER, STUB_PATH)
            cron.__salt__['cmd.run_all'].assert_called_with("crontab /tmp",
                                                            runas=STUB_USER,
                                                            python_shell=False)

    def test_write_cr_file_v_foo_rh(self):
        '''
        Assert that write_cron_file_verbose() is called with the correct cron command and user: RedHat
          - If instance running with uid that doesn't match crontab user uid, run with -u flag
        '''
        with patch.dict(cron.__grains__, {'os_family': 'Redhat'}), \
                patch.dict(cron.__salt__, {'cmd.run_all': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                            MagicMock(return_value=False)):
            cron.write_cron_file_verbose('foo', STUB_PATH)
            cron.__salt__['cmd.run_all'].assert_called_with("crontab -u foo /tmp",
                                                            python_shell=False)

    def test_write_cr_file_v_root_sol(self):
        '''
        Assert that write_cron_file_verbose() is called with the correct cron command and user: Solaris
          - Solaris should always run without a -u flag
        '''
        with patch.dict(cron.__grains__, {'os_family': 'Solaris'}), \
                patch.dict(cron.__salt__, {'cmd.run_all': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=True)):
            cron.write_cron_file_verbose(STUB_USER, STUB_PATH)
            cron.__salt__['cmd.run_all'].assert_called_with("crontab /tmp",
                                                            runas=STUB_USER,
                                                            python_shell=False)

    def test_write_cr_file_v_foo_sol(self):
        '''
        Assert that write_cron_file_verbose() is called with the correct cron command and user: Solaris
          - Solaris should always run without a -u flag
        '''
        with patch.dict(cron.__grains__, {'os_family': 'Solaris'}), \
                patch.dict(cron.__salt__, {'cmd.run_all': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=False)):
            cron.write_cron_file_verbose('foo', STUB_PATH)
            cron.__salt__['cmd.run_all'].assert_called_with("crontab /tmp",
                                                            runas='foo',
                                                            python_shell=False)

    def test_write_cr_file_v_root_aix(self):
        '''
        Assert that write_cron_file_verbose() is called with the correct cron command and user: AIX
          - AIX should always run without a -u flag
        '''
        with patch.dict(cron.__grains__, {'os_family': 'AIX'}), \
                patch.dict(cron.__salt__, {'cmd.run_all': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=True)):
            cron.write_cron_file_verbose(STUB_USER, STUB_PATH)
            cron.__salt__['cmd.run_all'].assert_called_with("crontab /tmp",
                                                            runas=STUB_USER,
                                                            python_shell=False)

    def test_write_cr_file_v_foo_aix(self):
        '''
        Assert that write_cron_file_verbose() is called with the correct cron command and user: AIX
          - AIX should always run without a -u flag
        '''
        with patch.dict(cron.__grains__, {'os_family': 'AIX'}), \
                patch.dict(cron.__salt__, {'cmd.run_all': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=False)):
            cron.write_cron_file_verbose('foo', STUB_PATH)
            cron.__salt__['cmd.run_all'].assert_called_with("crontab /tmp",
                                                            runas='foo',
                                                            python_shell=False)

    def test_raw_cron_root_redhat(self):
        '''
        Assert that raw_cron() is called with the correct cron command and user: RedHat
          - If instance running uid matches crontab user uid, runas STUB_USER without -u flag.
        '''
        with patch.dict(cron.__grains__, {'os_family': 'Redhat'}), \
                patch.dict(cron.__salt__, {'cmd.run_stdout': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=True)):
            cron.raw_cron(STUB_USER)
            cron.__salt__['cmd.run_stdout'].assert_called_with("crontab -l",
                                                               runas=STUB_USER,
                                                               ignore_retcode=True,
                                                               rstrip=False,
                                                               python_shell=False)

    def test_raw_cron_foo_redhat(self):
        '''
        Assert that raw_cron() is called with the correct cron command and user: RedHat
          - If instance running with uid that doesn't match crontab user uid, run with -u flag
        '''
        with patch.dict(cron.__grains__, {'os_family': 'Redhat'}), \
                patch.dict(cron.__salt__, {'cmd.run_stdout': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=False)):
            cron.raw_cron(STUB_USER)
            cron.__salt__['cmd.run_stdout'].assert_called_with("crontab -u root -l",
                                                               ignore_retcode=True,
                                                               rstrip=False,
                                                               python_shell=False)

    def test_raw_cron_root_solaris(self):
        '''
        Assert that raw_cron() is called with the correct cron command and user: Solaris
          - Solaris should always run without a -u flag
        '''
        with patch.dict(cron.__grains__, {'os_family': 'Solaris'}), \
                patch.dict(cron.__salt__, {'cmd.run_stdout': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=True)):
            cron.raw_cron(STUB_USER)
            cron.__salt__['cmd.run_stdout'].assert_called_with("crontab -l",
                                                               runas=STUB_USER,
                                                               ignore_retcode=True,
                                                               rstrip=False,
                                                               python_shell=False)

    def test_raw_cron_foo_solaris(self):
        '''
        Assert that raw_cron() is called with the correct cron command and user: Solaris
          - Solaris should always run without a -u flag
        '''
        with patch.dict(cron.__grains__, {'os_family': 'Solaris'}), \
                patch.dict(cron.__salt__, {'cmd.run_stdout': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=False)):
            cron.raw_cron(STUB_USER)
            cron.__salt__['cmd.run_stdout'].assert_called_with("crontab -l",
                                                               runas=STUB_USER,
                                                               ignore_retcode=True,
                                                               rstrip=False,
                                                               python_shell=False)

    def test_raw_cron_root_aix(self):
        '''
        Assert that raw_cron() is called with the correct cron command and user: AIX
          - AIX should always run without a -u flag
        '''
        with patch.dict(cron.__grains__, {'os_family': 'AIX'}), \
                patch.dict(cron.__salt__, {'cmd.run_stdout': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=True)):
            cron.raw_cron(STUB_USER)
            cron.__salt__['cmd.run_stdout'].assert_called_with("crontab -l",
                                                               runas=STUB_USER,
                                                               ignore_retcode=True,
                                                               rstrip=False,
                                                               python_shell=False)

    def test_raw_cron_foo_aix(self):
        '''
        Assert that raw_cron() is called with the correct cron command and user: AIX
          - AIX should always run without a -u flag
        '''
        with patch.dict(cron.__grains__, {'os_family': 'AIX'}), \
                patch.dict(cron.__salt__, {'cmd.run_stdout': MagicMock()}), \
                    patch('salt.modules.cron._check_instance_uid_match',
                          MagicMock(return_value=False)):
            cron.raw_cron(STUB_USER)
            cron.__salt__['cmd.run_stdout'].assert_called_with("crontab -l",
                                                               runas=STUB_USER,
                                                               ignore_retcode=True,
                                                               rstrip=False,
                                                               python_shell=False)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PsTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {cron: {}}

    def test__needs_change(self):
        self.assertTrue(cron._needs_change(True, False))

    def test__needs_change_random(self):
        '''
        Assert that if the new var is 'random' and old is '* that we return True
        '''
        self.assertTrue(cron._needs_change('*', 'random'))

    ## Still trying to figure this one out.
    # def test__render_tab(self):
    #     pass
    def test__get_cron_cmdstr(self):
        self.assertEqual('crontab /tmp', cron._get_cron_cmdstr(STUB_PATH))

    # Test get_cron_cmdstr() when user is added
    def test__get_cron_cmdstr_user(self):
        '''
        Passes if a user is added to crontab command
        '''
        self.assertEqual('crontab -u root /tmp', cron._get_cron_cmdstr(STUB_PATH, STUB_USER))

    def test__date_time_match(self):
        '''
        Passes if a match is found on all elements. Note the conversions to strings here!
        :return:
        '''
        self.assertTrue(cron._date_time_match(STUB_CRON_TIMESTAMP,
                                    minute=STUB_CRON_TIMESTAMP['minute'],
                                    hour=STUB_CRON_TIMESTAMP['hour'],
                                    daymonth=STUB_CRON_TIMESTAMP['daymonth'],
                                    dayweek=STUB_CRON_TIMESTAMP['dayweek']
                                    ))

    def test_list_tab(self):
        with patch('salt.modules.cron.raw_cron', new=MagicMock(return_value=STUB_SIMPLE_RAW_CRON)):
            self.assertDictEqual(STUB_SIMPLE_CRON_DICT, cron.list_tab('DUMMY_USER'))

    def test_set_special(self):
        with patch('salt.modules.cron._write_cron_lines') as write_cron_lines_mock, \
                patch('salt.modules.cron.list_tab', new=MagicMock(return_value=STUB_SIMPLE_CRON_DICT)):
            expected_write_call = call('DUMMY_USER',
                                       ['5 0 * * * /tmp/no_script.sh\n',
                                        '# Lines below here are managed by Salt, do not edit\n',
                                        '@hourly echo Hi!\n'])
            ret = cron.set_special('DUMMY_USER', '@hourly', 'echo Hi!')
            write_cron_lines_mock.assert_has_calls((expected_write_call,), any_order=True)

    def test__get_cron_date_time(self):
        ret = cron._get_cron_date_time(minute=STUB_CRON_TIMESTAMP['minute'],
                                    hour=STUB_CRON_TIMESTAMP['hour'],
                                    daymonth=STUB_CRON_TIMESTAMP['daymonth'],
                                    dayweek=STUB_CRON_TIMESTAMP['dayweek'],
                                    month=STUB_CRON_TIMESTAMP['month'])
        self.assertDictEqual(ret, STUB_CRON_TIMESTAMP)

    ## FIXME: More sophisticated _get_cron_date_time checks should be added here.

    def test_set_job(self):
        with patch.dict(cron.__grains__, {'os': None}), \
                patch('salt.modules.cron._write_cron_lines',
                      new=MagicMock(return_value={'retcode': False})), \
                              patch('salt.modules.cron.raw_cron',
                                    new=MagicMock(return_value=STUB_SIMPLE_RAW_CRON)):
            cron.set_job('DUMMY_USER', 1, 2, 3, 4, 5,
                         '/bin/echo NOT A DROID',
                         'WERE YOU LOOKING FOR ME?')
            expected_call = call('DUMMY_USER',
                                 ['5 0 * * * /tmp/no_script.sh\n',
                                  '# Lines below here are managed by Salt, do not edit\n',
                                  '# WERE YOU LOOKING FOR ME?\n',
                                  '1 2 3 4 5 /bin/echo NOT A DROID\n'])
            cron._write_cron_lines.call_args.assert_called_with(expected_call)

    def test_rm_job_is_absent(self):
        with patch.dict(cron.__grains__, {'os': None}), \
                patch('salt.modules.cron._write_cron_lines',
                      new=MagicMock(return_value={'retcode': False})), \
                              patch('salt.modules.cron.raw_cron',
                                    new=MagicMock(return_value=STUB_SIMPLE_RAW_CRON)):
            ret = cron.rm_job('DUMMY_USER', '/bin/echo NOT A DROID', 1, 2, 3, 4, 5)
            self.assertEqual('absent', ret)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import textwrap

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch

# Import Salt Libs
import salt.config
import salt.loader
import salt.states.boto_sqs as boto_sqs


@skipIf(NO_MOCK, NO_MOCK_REASON)
class BotoSqsTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.boto_sqs
    '''
    def setup_loader_modules(self):
        utils = salt.loader.utils(
            self.opts,
            whitelist=['boto3', 'yaml', 'args', 'systemd', 'path', 'platform'],
            context={})
        return {
            boto_sqs: {
                '__utils__': utils,
            }
        }

    @classmethod
    def setUpClass(cls):
        cls.opts = salt.config.DEFAULT_MINION_OPTS

    @classmethod
    def tearDownClass(cls):
        del cls.opts

    # 'present' function tests: 1

    def test_present(self):
        '''
        Test to ensure the SQS queue exists.
        '''
        name = 'mysqs'
        attributes = {'DelaySeconds': 20}
        base_ret = {'name': name, 'changes': {}}

        mock = MagicMock(
            side_effect=[{'result': b} for b in [False, False, True, True]],
        )
        mock_bool = MagicMock(return_value={'error': 'create error'})
        mock_attr = MagicMock(return_value={'result': {}})
        with patch.dict(boto_sqs.__salt__,
                        {'boto_sqs.exists': mock,
                         'boto_sqs.create': mock_bool,
                         'boto_sqs.get_attributes': mock_attr}):
            with patch.dict(boto_sqs.__opts__, {'test': False}):
                comt = ['Failed to create SQS queue {0}: create error'.format(
                    name,
                )]
                ret = base_ret.copy()
                ret.update({'result': False, 'comment': comt})
                self.assertDictEqual(boto_sqs.present(name), ret)

            with patch.dict(boto_sqs.__opts__, {'test': True}):
                comt = ['SQS queue {0} is set to be created.'.format(name)]
                ret = base_ret.copy()
                ret.update({
                    'result': None,
                    'comment': comt,
                    'pchanges': {'old': None, 'new': 'mysqs'},
                })
                self.assertDictEqual(boto_sqs.present(name), ret)
                diff = textwrap.dedent('''\
                    ---
                    +++
                    @@ -1 +1 @@
                    -{}
                    +DelaySeconds: 20

                ''').splitlines()
                # Difflib adds a trailing space after the +++/--- lines,
                # programatically add them back here. Having them in the test
                # file itself is not feasible since a few popular plugins for
                # vim will remove trailing whitespace.
                for idx in (0, 1):
                    diff[idx] += ' '
                diff = '\n'.join(diff)

                comt = [
                    'SQS queue mysqs present.',
                    'Attribute(s) DelaySeconds set to be updated:\n{0}'.format(
                        diff,
                    ),
                ]
                ret.update({
                    'comment': comt,
                    'pchanges': {'attributes': {'diff': diff}},
                })
                self.assertDictEqual(boto_sqs.present(name, attributes), ret)

            comt = ['SQS queue mysqs present.']
            ret = base_ret.copy()
            ret.update({'result': True, 'comment': comt})
            self.assertDictEqual(boto_sqs.present(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        '''
        Test to ensure the named sqs queue is deleted.
        '''
        name = 'test.example.com.'
        base_ret = {'name': name, 'changes': {}}

        mock = MagicMock(side_effect=[{'result': False}, {'result': True}])
        with patch.dict(boto_sqs.__salt__,
                        {'boto_sqs.exists': mock}):
            comt = ('SQS queue {0} does not exist in None.'.format(name))
            ret = base_ret.copy()
            ret.update({'result': True, 'comment': comt})
            self.assertDictEqual(boto_sqs.absent(name), ret)

            with patch.dict(boto_sqs.__opts__, {'test': True}):
                comt = ('SQS queue {0} is set to be removed.'.format(name))
                ret = base_ret.copy()
                ret.update({
                    'result': None,
                    'comment': comt,
                    'pchanges': {'old': name, 'new': None},
                })
                self.assertDictEqual(boto_sqs.absent(name), ret)



# -*- coding: utf-8 -*-
'''
unittests for table outputter
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase

# Import Salt Libs
import salt.output.table_out as table_out
import salt.utils.stringutils


class TableTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.output.table_out
    '''
    def setup_loader_modules(self):
        return {table_out: {}}

    # The test data should include unicode chars, and in Python 2 there should
    # be an example both of an encoded str type and an actual unicode type.
    # Since unicode_literals is imported, we will achieve the former using
    # salt.utils.stringutils.to_str and the latter by simply using a string
    # literal.
    data = [
        {'Food': salt.utils.stringutils.to_str('яйца, бекон, колбаса и спам'),
         'Price': 5.99},
        {'Food': 'спам, спам, спам, яйца и спам',
         'Price': 3.99},
    ]

    def test_output(self):
        ret = table_out.output(self.data)
        self.assertEqual(
            ret,
            ('    -----------------------------------------\n'
             '    |              Food             | Price |\n'
             '    -----------------------------------------\n'
             '    |  яйца, бекон, колбаса и спам  |  5.99 |\n'
             '    -----------------------------------------\n'
             '    | спам, спам, спам, яйца и спам |  3.99 |\n'
             '    -----------------------------------------')
        )



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

# Import Salt Libs
import salt.states.influxdb08_database as influxdb08_database


@skipIf(NO_MOCK, NO_MOCK_REASON)
class InfluxdbDatabaseTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.influxdb08_database
    '''
    def setup_loader_modules(self):
        return {influxdb08_database: {}}

    # 'present' function tests: 1

    def test_present(self):
        '''
        Test to ensure that the named database is present.
        '''
        name = 'salt'

        ret = {'name': name,
               'result': None,
               'comment': '',
               'changes': {}}

        mock = MagicMock(side_effect=[False, False, False, True])
        mock_t = MagicMock(side_effect=[True, False])
        with patch.dict(influxdb08_database.__salt__,
                        {'influxdb08.db_exists': mock,
                         'influxdb08.db_create': mock_t}):
            with patch.dict(influxdb08_database.__opts__, {'test': True}):
                comt = ('Database {0} is absent and needs to be created'
                        .format(name))
                ret.update({'comment': comt})
                self.assertDictEqual(influxdb08_database.present(name), ret)

            with patch.dict(influxdb08_database.__opts__, {'test': False}):
                comt = ('Database {0} has been created'.format(name))
                ret.update({'comment': comt, 'result': True,
                            'changes': {'salt': 'Present'}})
                self.assertDictEqual(influxdb08_database.present(name), ret)

                comt = ('Failed to create database {0}'.format(name))
                ret.update({'comment': comt, 'result': False, 'changes': {}})
                self.assertDictEqual(influxdb08_database.present(name), ret)

            comt = ('Database {0} is already present, so cannot be created'
                    .format(name))
            ret.update({'comment': comt, 'result': True})
            self.assertDictEqual(influxdb08_database.present(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        '''
        Test to ensure that the named database is absent.
        '''
        name = 'salt'

        ret = {'name': name,
               'result': None,
               'comment': '',
               'changes': {}}

        mock = MagicMock(side_effect=[True, True, True, False])
        mock_t = MagicMock(side_effect=[True, False])
        with patch.dict(influxdb08_database.__salt__,
                        {'influxdb08.db_exists': mock,
                         'influxdb08.db_remove': mock_t}):
            with patch.dict(influxdb08_database.__opts__, {'test': True}):
                comt = ('Database {0} is present and needs to be removed'
                        .format(name))
                ret.update({'comment': comt})
                self.assertDictEqual(influxdb08_database.absent(name), ret)

            with patch.dict(influxdb08_database.__opts__, {'test': False}):
                comt = ('Database {0} has been removed'.format(name))
                ret.update({'comment': comt, 'result': True,
                            'changes': {'salt': 'Absent'}})
                self.assertDictEqual(influxdb08_database.absent(name), ret)

                comt = ('Failed to remove database {0}'.format(name))
                ret.update({'comment': comt, 'result': False, 'changes': {}})
                self.assertDictEqual(influxdb08_database.absent(name), ret)

            comt = ('Database {0} is not present, so it cannot be removed'
                    .format(name))
            ret.update({'comment': comt, 'result': True})
            self.assertDictEqual(influxdb08_database.absent(name), ret)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import shutil
import threading
import time
import logging

# Import Salt Testing Libs
from tests.support.case import SSHCase
from tests.support.helpers import flaky
from tests.support.paths import TMP

# Import Salt Libs
from salt.ext import six
from salt.ext.six.moves import range  # pylint: disable=redefined-builtin

SSH_SLS = 'ssh_state_tests'
SSH_SLS_FILE = '/tmp/test'

log = logging.getLogger(__name__)


class SSHStateTest(SSHCase):
    '''
    testing the state system with salt-ssh
    '''
    def _check_dict_ret(self, ret, val, exp_ret, equal=True):
        self.assertIsInstance(ret, dict)
        for key, value in ret.items():
            self.assertIsInstance(value, dict)
            if equal:
                self.assertEqual(value[val], exp_ret)
            else:
                self.assertNotEqual(value[val], exp_ret)

    def _check_request(self, empty=False):
        check = self.run_function('state.check_request', wipe=False)
        if empty:
            self.assertFalse(bool(check), 'bool({0}) is not False'.format(check))
        else:
            self._check_dict_ret(ret=check['default']['test_run']['local']['return'],
                       val='__sls__', exp_ret=SSH_SLS)

    def test_state_apply(self):
        '''
        test state.apply with salt-ssh
        '''
        ret = self.run_function('state.apply', [SSH_SLS])
        self._check_dict_ret(ret=ret, val='__sls__', exp_ret=SSH_SLS)

        check_file = self.run_function('file.file_exists', ['/tmp/test'])
        self.assertTrue(check_file)

    def test_state_sls_id(self):
        '''
        test state.sls_id with salt-ssh
        '''
        # check state.sls_id with test=True
        ret = self.run_function('state.sls_id', ['ssh-file-test', SSH_SLS,
                                                 'test=True'])
        self._check_dict_ret(ret=ret, val='comment',
                             exp_ret='The file /tmp/test is set to be changed')

        # check state.sls_id without test=True
        ret = self.run_function('state.sls_id', ['ssh-file-test', SSH_SLS])
        self._check_dict_ret(ret=ret, val='__sls__', exp_ret=SSH_SLS)

        # make sure the other id in the state was not run
        self._check_dict_ret(ret=ret, val='__id__',
                             exp_ret='second_id', equal=False)

        check_file = self.run_function('file.file_exists', ['/tmp/test'])
        self.assertTrue(check_file)

    def test_state_sls_wrong_id(self):
        '''
        test state.sls_id when id does not exist
        '''
        # check state.sls_id with test=True
        ret = self.run_function('state.sls_id', ['doesnotexist', SSH_SLS])
        assert 'No matches for ID' in ret

    def test_state_show_sls(self):
        '''
        test state.show_sls with salt-ssh
        '''
        ret = self.run_function('state.show_sls', [SSH_SLS])
        self._check_dict_ret(ret=ret, val='__sls__', exp_ret=SSH_SLS)

        check_file = self.run_function('file.file_exists', [SSH_SLS_FILE], wipe=False)
        self.assertFalse(check_file)

    def test_state_show_top(self):
        '''
        test state.show_top with salt-ssh
        '''
        ret = self.run_function('state.show_top')
        self.assertEqual(ret, {'base': ['core', 'master_tops_test']})

    def test_state_single(self):
        '''
        state.single with salt-ssh
        '''
        ret_out = {'name': 'itworked',
                   'result': True,
                   'comment': 'Success!'}

        single = self.run_function('state.single',
                                   ['test.succeed_with_changes name=itworked'])

        self.assertIsInstance(single, dict)
        for key, value in six.iteritems(single):
            self.assertIsInstance(value, dict)
            self.assertEqual(value['name'], ret_out['name'])
            self.assertEqual(value['result'], ret_out['result'])
            self.assertEqual(value['comment'], ret_out['comment'])

    def test_show_highstate(self):
        '''
        state.show_highstate with salt-ssh
        '''
        high = self.run_function('state.show_highstate')
        destpath = os.path.join(TMP, 'testfile')
        self.assertIsInstance(high, dict)
        self.assertIn(destpath, high)
        self.assertEqual(high[destpath]['__env__'], 'base')

    def test_state_high(self):
        '''
        state.high with salt-ssh
        '''
        ret_out = {'name': 'itworked',
                   'result': True,
                   'comment': 'Success!'}

        high = self.run_function('state.high',
                                 ['"{"itworked": {"test": ["succeed_with_changes"]}}"'])

        self.assertIsInstance(high, dict)
        for key, value in six.iteritems(high):
            self.assertIsInstance(value, dict)
            self.assertEqual(value['name'], ret_out['name'])
            self.assertEqual(value['result'], ret_out['result'])
            self.assertEqual(value['comment'], ret_out['comment'])

    def test_show_lowstate(self):
        '''
        state.show_lowstate with salt-ssh
        '''
        low = self.run_function('state.show_lowstate')
        self.assertIsInstance(low, list)
        self.assertIsInstance(low[0], dict)

    def test_state_low(self):
        '''
        state.low with salt-ssh
        '''
        ret_out = {'name': 'itworked',
                   'result': True,
                   'comment': 'Success!'}

        low = self.run_function(
                'state.low',
                ['"{"state": "test", "fun": "succeed_with_changes", "name": "itworked"}"'])

        self.assertIsInstance(low, dict)
        for key, value in six.iteritems(low):
            self.assertIsInstance(value, dict)
            self.assertEqual(value['name'], ret_out['name'])
            self.assertEqual(value['result'], ret_out['result'])
            self.assertEqual(value['comment'], ret_out['comment'])

    def test_state_request_check_clear(self):
        '''
        test state.request system with salt-ssh
        while also checking and clearing request
        '''
        request = self.run_function('state.request', [SSH_SLS], wipe=False)
        self._check_dict_ret(ret=request, val='__sls__', exp_ret=SSH_SLS)

        self._check_request()

        clear = self.run_function('state.clear_request', wipe=False)
        self._check_request(empty=True)

    def test_state_run_request(self):
        '''
        test state.request system with salt-ssh
        while also running the request later
        '''
        request = self.run_function('state.request', [SSH_SLS], wipe=False)
        self._check_dict_ret(ret=request, val='__sls__', exp_ret=SSH_SLS)

        run = self.run_function('state.run_request', wipe=False)

        check_file = self.run_function('file.file_exists', [SSH_SLS_FILE], wipe=False)
        self.assertTrue(check_file)

    @flaky
    def test_state_running(self):
        '''
        test state.running with salt-ssh
        '''
        def _run_in_background():
            self.run_function('state.sls', ['running'], wipe=False)

        bg_thread = threading.Thread(target=_run_in_background)
        bg_thread.start()

        expected = 'The function "state.pkg" is running as'
        state_ret = []
        for _ in range(30):
            time.sleep(5)
            get_sls = self.run_function('state.running', wipe=False)
            state_ret.append(get_sls)
            if expected in ' '.join(get_sls):
                # We found the expected return
                break
        else:
            self.fail(
                    'Did not find \'{0}\' in state.running return: {1}'.format(expected, state_ret)
            )

        # make sure we wait until the earlier state is complete
        future = time.time() + 120
        while True:
            if expected not in ' '.join(self.run_function('state.running', wipe=False)):
                break
            if time.time() > future:
                self.fail('state.pkg is still running overtime. Test did not clean up correctly.')

    def tearDown(self):
        '''
        make sure to clean up any old ssh directories
        '''
        salt_dir = self.run_function('config.get', ['thin_dir'], wipe=False)
        self.assertIsInstance(salt_dir, six.string_types)
        if os.path.exists(salt_dir):
            shutil.rmtree(salt_dir)

        if os.path.exists(SSH_SLS_FILE):
            os.remove(SSH_SLS_FILE)



# -*- coding: utf-8 -*-
'''
Tests for the service state
'''
# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import re

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.helpers import destructiveTest
from tests.support.mixins import SaltReturnAssertsMixin

# Import salt libs
import salt.utils.path
import salt.utils.platform

INIT_DELAY = 5


@destructiveTest
class ServiceTest(ModuleCase, SaltReturnAssertsMixin):
    '''
    Validate the service state
    '''
    def setUp(self):
        self.service_name = 'cron'
        cmd_name = 'crontab'
        os_family = self.run_function('grains.get', ['os_family'])
        os_release = self.run_function('grains.get', ['osrelease'])
        self.stopped = False
        self.running = True
        if os_family == 'RedHat':
            self.service_name = 'crond'
        elif os_family == 'Arch':
            self.service_name = 'sshd'
            cmd_name = 'systemctl'
        elif os_family == 'MacOS':
            self.service_name = 'org.ntp.ntpd'
            if int(os_release.split('.')[1]) >= 13:
                self.service_name = 'com.apple.AirPlayXPCHelper'
            self.stopped = ''
            self.running = '[0-9]'
        elif os_family == 'Windows':
            self.service_name = 'Spooler'

        self.pre_srv_enabled = True if self.service_name in self.run_function('service.get_enabled') else False
        self.post_srv_disable = False
        if not self.pre_srv_enabled:
            self.run_function('service.enable', name=self.service_name)
            self.post_srv_disable = True

        if os_family != 'Windows' and salt.utils.path.which(cmd_name) is None:
            self.skipTest('{0} is not installed'.format(cmd_name))

    def tearDown(self):
        if self.post_srv_disable:
            self.run_function('service.disable', name=self.service_name)

    def check_service_status(self, exp_return):
        '''
        helper method to check status of service
        '''
        check_status = self.run_function('service.status',
                                         name=self.service_name)

        try:
            if not re.match(exp_return, check_status):
                self.fail('status of service is not returning correctly')
        except TypeError:
            if check_status is not exp_return:
                self.fail('status of service is not returning correctly')

    def test_service_running(self):
        '''
        test service.running state module
        '''
        if self.run_function('service.status', name=self.service_name):
            stop_service = self.run_function('service.stop', name=self.service_name)
            self.assertTrue(stop_service)
        self.check_service_status(self.stopped)

        if salt.utils.platform.is_darwin():
            # make sure the service is enabled on macosx
            enable = self.run_function('service.enable', name=self.service_name)

        start_service = self.run_state('service.running',
                                       name=self.service_name)
        self.assertTrue(start_service)
        self.check_service_status(self.running)

    def test_service_dead(self):
        '''
        test service.dead state module
        '''
        start_service = self.run_state('service.running',
                                       name=self.service_name)
        self.assertSaltTrueReturn(start_service)
        self.check_service_status(self.running)

        ret = self.run_state('service.dead', name=self.service_name)
        self.assertSaltTrueReturn(ret)
        self.check_service_status(self.stopped)

    def test_service_dead_init_delay(self):
        '''
        test service.dead state module with init_delay arg
        '''
        start_service = self.run_state('service.running',
                                       name=self.service_name)
        self.assertSaltTrueReturn(start_service)
        self.check_service_status(self.running)

        ret = self.run_state('service.dead', name=self.service_name,
                             init_delay=INIT_DELAY)
        self.assertSaltTrueReturn(ret)
        self.check_service_status(self.stopped)



# -*- coding: utf-8 -*-
'''
tests.unit.utils.test_configparser
==================================

Test the funcs in the custom parsers in salt.utils.configparser
'''
# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals
import copy
import errno
import logging
import os

log = logging.getLogger(__name__)

# Import Salt Testing Libs
from tests.support.unit import TestCase
from tests.support.paths import TMP

# Import salt libs
import salt.utils.files
import salt.utils.stringutils
import salt.utils.configparser
import salt.utils.platform
from salt.ext import six

# The user.name param here is intentionally indented with spaces instead of a
# tab to test that we properly load a file with mixed indentation.
ORIG_CONFIG = '''[user]
        name = Артём Анисимов
\temail = foo@bar.com
[remote "origin"]
\turl = https://github.com/terminalmage/salt.git
\tfetch = +refs/heads/*:refs/remotes/origin/*
\tpushurl = git@github.com:terminalmage/salt.git
[color "diff"]
\told = 196
\tnew = 39
[core]
\tpager = less -R
\trepositoryformatversion = 0
\tfilemode = true
\tbare = false
\tlogallrefupdates = true
[alias]
\tmodified = ! git status --porcelain | awk 'match($1, "M"){print $2}'
\tgraph = log --all --decorate --oneline --graph
\thist = log --pretty=format:\\"%h %ad | %s%d [%an]\\" --graph --date=short
[http]
\tsslverify = false'''.split('\n')


class TestGitConfigParser(TestCase):
    '''
    Tests for salt.utils.configparser.GitConfigParser
    '''
    maxDiff = None
    orig_config = os.path.join(TMP, 'test_gitconfig.orig')
    new_config = os.path.join(TMP, 'test_gitconfig.new')
    remote = 'remote "origin"'

    def tearDown(self):
        del self.conf
        try:
            os.remove(self.new_config)
        except OSError as exc:
            if exc.errno != errno.ENOENT:
                raise

    def setUp(self):
        if not os.path.exists(self.orig_config):
            with salt.utils.files.fopen(self.orig_config, 'wb') as fp_:
                fp_.write(
                    salt.utils.stringutils.to_bytes(
                        os.linesep.join(ORIG_CONFIG)
                    )
                )
        self.conf = salt.utils.configparser.GitConfigParser()
        with salt.utils.files.fopen(self.orig_config, 'rb') as fp:
            self.conf._read(fp, self.orig_config)

    @classmethod
    def tearDownClass(cls):
        try:
            os.remove(cls.orig_config)
        except OSError as exc:
            if exc.errno != errno.ENOENT:
                raise

    @staticmethod
    def fix_indent(lines):
        '''
        Fixes the space-indented 'user' line, because when we write the config
        object to a file space indentation will be replaced by tab indentation.
        '''
        ret = copy.copy(lines)
        for i, _ in enumerate(ret):
            if ret[i].startswith(salt.utils.configparser.GitConfigParser.SPACEINDENT):
                ret[i] = ret[i].replace(salt.utils.configparser.GitConfigParser.SPACEINDENT, '\t')
        return ret

    @staticmethod
    def get_lines(path):
        with salt.utils.files.fopen(path, 'rb') as fp_:
            return salt.utils.stringutils.to_unicode(fp_.read()).splitlines()

    def _test_write(self, mode):
        kwargs = {'mode': mode}
        if six.PY3 and salt.utils.platform.is_windows() and 'b' not in mode:
            kwargs['encoding'] = 'utf-8'
        with salt.utils.files.fopen(self.new_config, **kwargs) as fp_:
            self.conf.write(fp_)
        self.assertEqual(
            self.get_lines(self.new_config),
            self.fix_indent(ORIG_CONFIG)
        )

    def test_get(self):
        '''
        Test getting an option's value
        '''
        # Numeric values should be loaded as strings
        self.assertEqual(self.conf.get('color "diff"', 'old'), '196')
        # Complex strings should be loaded with their literal quotes and
        # slashes intact
        self.assertEqual(
            self.conf.get('alias', 'modified'),
            """! git status --porcelain | awk 'match($1, "M"){print $2}'"""
        )
        # future lint: disable=non-unicode-string
        self.assertEqual(
            self.conf.get('alias', 'hist'),
            salt.utils.stringutils.to_unicode(
                r"""log --pretty=format:\"%h %ad | %s%d [%an]\" --graph --date=short"""
            )
        )
        # future lint: enable=non-unicode-string

    def test_read_space_indent(self):
        '''
        Test that user.name was successfully loaded despite being indented
        using spaces instead of a tab. Additionally, this tests that the value
        was loaded as a unicode type on PY2.
        '''
        self.assertEqual(self.conf.get('user', 'name'), u'Артём Анисимов')

    def test_set_new_option(self):
        '''
        Test setting a new option in an existing section
        '''
        self.conf.set('http', 'useragent', 'myawesomeagent')
        self.assertEqual(self.conf.get('http', 'useragent'), 'myawesomeagent')

    def test_add_section(self):
        '''
        Test adding a section and adding an item to that section
        '''
        self.conf.add_section('foo')
        self.conf.set('foo', 'bar', 'baz')
        self.assertEqual(self.conf.get('foo', 'bar'), 'baz')

    def test_replace_option(self):
        '''
        Test replacing an existing option
        '''
        # We're also testing the normalization of key names, here. Setting
        # "sslVerify" should actually set an "sslverify" option.
        self.conf.set('http', 'sslVerify', 'true')
        self.assertEqual(self.conf.get('http', 'sslverify'), 'true')

    def test_set_multivar(self):
        '''
        Test setting a multivar and then writing the resulting file
        '''
        orig_refspec = '+refs/heads/*:refs/remotes/origin/*'
        new_refspec = '+refs/tags/*:refs/tags/*'
        # Make sure that the original value is a string
        self.assertEqual(
            self.conf.get(self.remote, 'fetch'),
            orig_refspec
        )
        # Add another refspec
        self.conf.set_multivar(self.remote, 'fetch', new_refspec)
        # The value should now be a list
        self.assertEqual(
            self.conf.get(self.remote, 'fetch'),
            [orig_refspec, new_refspec]
        )
        # Write the config object to a file
        with salt.utils.files.fopen(self.new_config, 'wb') as fp_:
            self.conf.write(fp_)
        # Confirm that the new file was written correctly
        expected = self.fix_indent(ORIG_CONFIG)
        expected.insert(6, '\tfetch = %s' % new_refspec)  # pylint: disable=string-substitution-usage-error
        self.assertEqual(self.get_lines(self.new_config), expected)

    def test_remove_option(self):
        '''
        test removing an option, including all items from a multivar
        '''
        for item in ('fetch', 'pushurl'):
            self.conf.remove_option(self.remote, item)
            # To confirm that the option is now gone, a get should raise an
            # NoOptionError exception.
            self.assertRaises(
                salt.utils.configparser.NoOptionError,
                self.conf.get,
                self.remote,
                item)

    def test_remove_option_regexp(self):
        '''
        test removing an option, including all items from a multivar
        '''
        orig_refspec = '+refs/heads/*:refs/remotes/origin/*'
        new_refspec_1 = '+refs/tags/*:refs/tags/*'
        new_refspec_2 = '+refs/foo/*:refs/foo/*'
        # First, add both refspecs
        self.conf.set_multivar(self.remote, 'fetch', new_refspec_1)
        self.conf.set_multivar(self.remote, 'fetch', new_refspec_2)
        # Make sure that all three values are there
        self.assertEqual(
            self.conf.get(self.remote, 'fetch'),
            [orig_refspec, new_refspec_1, new_refspec_2]
        )
        # If the regex doesn't match, no items should be removed
        self.assertFalse(
            self.conf.remove_option_regexp(
                self.remote,
                'fetch',
                salt.utils.stringutils.to_unicode(r'\d{7,10}')  # future lint: disable=non-unicode-string
            )
        )
        # Make sure that all three values are still there (since none should
        # have been removed)
        self.assertEqual(
            self.conf.get(self.remote, 'fetch'),
            [orig_refspec, new_refspec_1, new_refspec_2]
        )
        # Remove one of the values
        self.assertTrue(
            self.conf.remove_option_regexp(self.remote, 'fetch', 'tags'))
        # Confirm that the value is gone
        self.assertEqual(
            self.conf.get(self.remote, 'fetch'),
            [orig_refspec, new_refspec_2]
        )
        # Remove the other one we added earlier
        self.assertTrue(
            self.conf.remove_option_regexp(self.remote, 'fetch', 'foo'))
        # Since the option now only has one value, it should be a string
        self.assertEqual(self.conf.get(self.remote, 'fetch'), orig_refspec)
        # Remove the last remaining option
        self.assertTrue(
            self.conf.remove_option_regexp(self.remote, 'fetch', 'heads'))
        # Trying to do a get now should raise an exception
        self.assertRaises(
            salt.utils.configparser.NoOptionError,
            self.conf.get,
            self.remote,
            'fetch')

    def test_write(self):
        '''
        Test writing using non-binary filehandle
        '''
        self._test_write(mode='w')

    def test_write_binary(self):
        '''
        Test writing using binary filehandle
        '''
        self._test_write(mode='wb')



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.helpers import (
    destructiveTest,
    flaky,
    requires_network,
    requires_salt_modules,
)
from tests.support.unit import skipIf

# Import Salt libs
import salt.utils.pkg
import salt.utils.platform


@flaky
class PkgModuleTest(ModuleCase, SaltReturnAssertsMixin):
    '''
    Validate the pkg module
    '''
    def setUp(self):
        if salt.utils.platform.is_windows():
            self.run_function('pkg.refresh_db')

        os_release = self.run_function('grains.get', ['osrelease'])

        if salt.utils.platform.is_darwin() and int(os_release.split('.')[1]) >= 13:
            self.pkg = 'wget'
        elif salt.utils.platform.is_windows():
            self.pkg = 'putty'
        else:
            self.pkg = 'htop'

    def test_list(self):
        '''
        verify that packages are installed
        '''
        ret = self.run_function('pkg.list_pkgs')
        self.assertNotEqual(len(ret.keys()), 0)

    @requires_salt_modules('pkg.version_cmp')
    def test_version_cmp(self):
        '''
        test package version comparison on supported platforms
        '''
        func = 'pkg.version_cmp'
        os_family = self.run_function('grains.item', ['os_family'])['os_family']
        if os_family == 'Debian':
            lt = ['0.2.4-0ubuntu1', '0.2.4.1-0ubuntu1']
            eq = ['0.2.4-0ubuntu1', '0.2.4-0ubuntu1']
            gt = ['0.2.4.1-0ubuntu1', '0.2.4-0ubuntu1']

            self.assertEqual(self.run_function(func, lt), -1)
            self.assertEqual(self.run_function(func, eq), 0)
            self.assertEqual(self.run_function(func, gt), 1)
        elif os_family == 'Suse':
            lt = ['2.3.0-1', '2.3.1-15.1']
            eq = ['2.3.1-15.1', '2.3.1-15.1']
            gt = ['2.3.2-15.1', '2.3.1-15.1']

            self.assertEqual(self.run_function(func, lt), -1)
            self.assertEqual(self.run_function(func, eq), 0)
            self.assertEqual(self.run_function(func, gt), 1)
        else:
            self.skipTest('{0} is unavailable on {1}'.format(func, os_family))

    @requires_salt_modules('pkg.mod_repo', 'pkg.del_repo')
    @requires_network()
    @destructiveTest
    def test_mod_del_repo(self):
        '''
        test modifying and deleting a software repository
        '''
        os_grain = self.run_function('grains.item', ['os'])['os']
        repo = None

        try:
            if os_grain == 'Ubuntu':
                repo = 'ppa:otto-kesselgulasch/gimp-edge'
                uri = 'http://ppa.launchpad.net/otto-kesselgulasch/gimp-edge/ubuntu'
                ret = self.run_function('pkg.mod_repo', [repo, 'comps=main'])
                self.assertNotEqual(ret, {})
                ret = self.run_function('pkg.get_repo', [repo])
                self.assertEqual(ret['uri'], uri)
            elif os_grain == 'CentOS':
                major_release = int(
                    self.run_function(
                        'grains.item',
                        ['osmajorrelease']
                    )['osmajorrelease']
                )
                repo = 'saltstack'
                name = 'SaltStack repo for RHEL/CentOS {0}'.format(major_release)
                baseurl = 'http://repo.saltstack.com/yum/redhat/{0}/x86_64/latest/'.format(major_release)
                gpgkey = 'https://repo.saltstack.com/yum/rhel{0}/SALTSTACK-GPG-KEY.pub'.format(major_release)
                gpgcheck = 1
                enabled = 1
                ret = self.run_function(
                    'pkg.mod_repo',
                    [repo],
                    name=name,
                    baseurl=baseurl,
                    gpgkey=gpgkey,
                    gpgcheck=gpgcheck,
                    enabled=enabled,
                )
                # return data from pkg.mod_repo contains the file modified at
                # the top level, so use next(iter(ret)) to get that key
                self.assertNotEqual(ret, {})
                repo_info = ret[next(iter(ret))]
                self.assertIn(repo, repo_info)
                self.assertEqual(repo_info[repo]['baseurl'], baseurl)
                ret = self.run_function('pkg.get_repo', [repo])
                self.assertEqual(ret['baseurl'], baseurl)
        finally:
            if repo is not None:
                self.run_function('pkg.del_repo', [repo])

    @requires_salt_modules('pkg.owner')
    def test_owner(self):
        '''
        test finding the package owning a file
        '''
        func = 'pkg.owner'
        available = self.run_function('sys.doc', [func])

        if available:
            ret = self.run_function(func, ['/bin/ls'])
            self.assertNotEqual(len(ret), 0)
        else:
            os_grain = self.run_function('grains.item', ['os'])['os']
            self.skipTest('{0} is unavailable on {1}'.format(func, os_grain))

    @requires_network()
    @destructiveTest
    def test_install_remove(self):
        '''
        successfully install and uninstall a package
        '''
        version = self.run_function('pkg.version', [self.pkg])

        def test_install():
            install_ret = self.run_function('pkg.install', [self.pkg])
            self.assertIn(self.pkg, install_ret)

        def test_remove():
            remove_ret = self.run_function('pkg.remove', [self.pkg])
            self.assertIn(self.pkg, remove_ret)

        if version and isinstance(version, dict):
            version = version[self.pkg]

        if version:
            test_remove()
            test_install()
        else:
            test_install()
            test_remove()

    @skipIf(salt.utils.platform.is_windows(), "Skip on windows")
    @requires_salt_modules('pkg.hold', 'pkg.unhold')
    @requires_network()
    @destructiveTest
    def test_hold_unhold(self):
        '''
        test holding and unholding a package
        '''
        os_family = self.run_function('grains.item', ['os_family'])['os_family']
        available = self.run_function('sys.doc', ['pkg.hold'])
        version_lock = None
        lock_pkg = 'yum-plugin-versionlock'

        if available:
            self.run_function('pkg.install', [self.pkg])
            if os_family == 'RedHat':
                version_lock = self.run_function('pkg.version', [lock_pkg])
                if not version_lock:
                    self.run_function('pkg.install', [lock_pkg])

            hold_ret = self.run_function('pkg.hold', [self.pkg])
            self.assertIn(self.pkg, hold_ret)
            self.assertTrue(hold_ret[self.pkg]['result'])

            unhold_ret = self.run_function('pkg.unhold', [self.pkg])
            self.assertIn(self.pkg, unhold_ret)
            self.assertTrue(unhold_ret[self.pkg]['result'])

            if os_family == 'RedHat':
                if not version_lock:
                    self.run_function('pkg.remove', [lock_pkg])
            self.run_function('pkg.remove', [self.pkg])

        else:
            os_grain = self.run_function('grains.item', ['os'])['os']
            self.skipTest('{0} is unavailable on {1}'.format('pkg.hold', os_grain))

    @requires_network()
    @destructiveTest
    def test_refresh_db(self):
        '''
        test refreshing the package database
        '''
        func = 'pkg.refresh_db'
        os_family = self.run_function('grains.item', ['os_family'])['os_family']

        rtag = salt.utils.pkg.rtag(self.minion_opts)
        salt.utils.pkg.write_rtag(self.minion_opts)
        self.assertTrue(os.path.isfile(rtag))

        if os_family == 'RedHat':
            ret = self.run_function(func)
            self.assertIn(ret, (True, None))
        elif os_family == 'Suse':
            ret = self.run_function(func)
            if not isinstance(ret, dict):
                self.skipTest('Upstream repo did not return coherent results. Skipping test.')
            self.assertNotEqual(ret, {})
        elif os_family == 'Debian':
            ret = self.run_function(func)
            if not isinstance(ret, dict):
                self.skipTest('{0} encountered an error: {1}'.format(func, ret))
            self.assertNotEqual(ret, {})
            if not isinstance(ret, dict):
                self.skipTest('Upstream repo did not return coherent results. Skipping test.')
            for source, state in ret.items():
                self.assertIn(state, (True, False, None))
        else:
            os_grain = self.run_function('grains.item', ['os'])['os']
            self.skipTest('{0} is unavailable on {1}'.format(func, os_grain))

        self.assertFalse(os.path.isfile(rtag))

    @requires_salt_modules('pkg.info_installed')
    def test_pkg_info(self):
        '''
        Test returning useful information on Ubuntu systems.
        '''
        func = 'pkg.info_installed'
        os_family = self.run_function('grains.item', ['os_family'])['os_family']

        if os_family == 'Debian':
            ret = self.run_function(func, ['bash-completion', 'dpkg'])
            keys = ret.keys()
            self.assertIn('bash-completion', keys)
            self.assertIn('dpkg', keys)
        elif os_family == 'RedHat':
            ret = self.run_function(func, ['rpm', 'bash'])
            keys = ret.keys()
            self.assertIn('rpm', keys)
            self.assertIn('bash', keys)
        elif os_family == 'Suse':
            ret = self.run_function(func, ['less', 'zypper'])
            keys = ret.keys()
            self.assertIn('less', keys)
            self.assertIn('zypper', keys)

    @requires_network()
    @destructiveTest
    @skipIf(salt.utils.platform.is_windows(), 'pkg.upgrade not available on Windows')
    def test_pkg_upgrade_has_pending_upgrades(self):
        '''
        Test running a system upgrade when there are packages that need upgrading
        '''
        func = 'pkg.upgrade'
        os_family = self.run_function('grains.item', ['os_family'])['os_family']

        # First make sure that an up-to-date copy of the package db is available
        self.run_function('pkg.refresh_db')

        if os_family == 'Suse':
            # This test assumes that there are multiple possible versions of a
            # package available. That makes it brittle if you pick just one
            # target, as changes in the available packages will break the test.
            # Therefore, we'll choose from several packages to make sure we get
            # one that is suitable for this test.
            packages = ('hwinfo', 'avrdude', 'diffoscope', 'vim')
            available = self.run_function('pkg.list_repo_pkgs', packages)

            for package in packages:
                try:
                    new, old = available[package][:2]
                except (KeyError, ValueError):
                    # Package not available, or less than 2 versions
                    # available. This is not a suitable target.
                    continue
                else:
                    target = package
                    break
            else:
                # None of the packages have more than one version available, so
                # we need to find new package(s). pkg.list_repo_pkgs can be
                # used to get an overview of the available packages. We should
                # try to find packages with few dependencies and small download
                # sizes, to keep this test from taking longer than necessary.
                self.fail('No suitable package found for this test')

            # Make sure we have the 2nd-oldest available version installed
            ret = self.run_function('pkg.install', [target], version=old)
            if not isinstance(ret, dict):
                if ret.startswith('ERROR'):
                    self.skipTest(
                        'Could not install older {0} to complete '
                        'test.'.format(target)
                    )

            # Run a system upgrade, which should catch the fact that the
            # targeted package needs upgrading, and upgrade it.
            ret = self.run_function(func)

            # The changes dictionary should not be empty.
            if 'changes' in ret:
                self.assertIn(target, ret['changes'])
            else:
                self.assertIn(target, ret)
        else:
            ret = self.run_function('pkg.list_upgrades')
            if ret == '' or ret == {}:
                self.skipTest('No updates available for this machine.  Skipping pkg.upgrade test.')
            else:
                args = []
                if os_family == 'Debian':
                    args = ['dist_upgrade=True']
                ret = self.run_function(func, args)
                self.assertNotEqual(ret, {})

    @destructiveTest
    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
    @skipIf(salt.utils.platform.is_darwin(), 'minion is mac')
    def test_pkg_latest_version(self):
        '''
        Check that pkg.latest_version returns the latest version of the uninstalled package.
        The package is not installed. Only the package version is checked.
        '''
        grains = self.run_function('grains.items')
        remove = False
        if salt.utils.platform.is_windows():
            cmd_info = self.run_function('pkg.version', [self.pkg])
            remove = False if cmd_info == '' else True
        else:
            cmd_info = self.run_function('pkg.info_installed', [self.pkg])
            if cmd_info != 'ERROR: package {0} is not installed'.format(self.pkg):
                remove = True

        # remove package if its installed
        if remove:
            self.run_function('pkg.remove', [self.pkg])

        cmd_pkg = []
        if grains['os_family'] == 'RedHat':
            cmd_pkg = self.run_function('cmd.run', ['yum list {0}'.format(self.pkg)])
        elif salt.utils.platform.is_windows():
            cmd_pkg = self.run_function('pkg.list_available', [self.pkg])
        elif grains['os_family'] == 'Debian':
            cmd_pkg = self.run_function('cmd.run', ['apt list {0}'.format(self.pkg)])
        elif grains['os_family'] == 'Arch':
            cmd_pkg = self.run_function('cmd.run', ['pacman -Si {0}'.format(self.pkg)])
        elif grains['os_family'] == 'Suse':
            cmd_pkg = self.run_function('cmd.run', ['zypper info {0}'.format(self.pkg)])
        pkg_latest = self.run_function('pkg.latest_version', [self.pkg])
        self.assertIn(pkg_latest, cmd_pkg)



# -*- coding: utf-8 -*-
'''
unittests for yaml outputter
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase
from tests.support.mock import patch

# Import Salt Libs
import salt.output.yaml_out as yaml


class YamlTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.output.json_out
    '''
    def setup_loader_modules(self):
        return {yaml: {}}

    def setUp(self):
        self.data = {'test': 'two', 'example': 'one'}
        self.addCleanup(delattr, self, 'data')

    def test_default_output(self):
        ret = yaml.output(self.data)
        expect = 'example: one\ntest: two\n'
        self.assertEqual(expect, ret)

    def test_negative_int_output(self):
        with patch.dict(yaml.__opts__, {'output_indent': -1}):
            ret = yaml.output(self.data)
            expect = '{example: one, test: two}\n'
            self.assertEqual(expect, ret)



# -*- coding: utf-8 -*-
'''
    tests.integration.shell.auth
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import logging
try:
    import pwd
    import grp
except ImportError:
    pwd, grp = None, None
import random

# Import Salt Testing libs
from tests.support.case import ShellCase
from tests.support.unit import skipIf
from tests.support.helpers import destructiveTest, skip_if_not_root

# Import Salt libs
import salt.utils.platform
from salt.utils.pycrypto import gen_hash

# Import 3rd-party libs
from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin

log = logging.getLogger(__name__)


def gen_password():
    '''
    generate a password and hash it
    '''
    alphabet = ('abcdefghijklmnopqrstuvwxyz'
                '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ')
    password = ''
    # generate password
    for _ in range(20):
        next_index = random.randrange(len(alphabet))
        password += alphabet[next_index]

    # hash the password
    hashed_pwd = gen_hash('salt', password, 'sha512')

    return (password, hashed_pwd)


@skip_if_not_root
@skipIf(pwd is None, 'Skip if no pwd module exists')
@destructiveTest
class AuthTest(ShellCase):
    '''
    Test auth mechanisms
    '''

    _call_binary_ = 'salt'

    userA = 'saltdev'
    userB = 'saltadm'
    group = 'saltops'

    def setUp(self):
        for user in (self.userA, self.userB):
            try:
                if salt.utils.platform.is_darwin() and user not in str(self.run_call('user.list_users')):
                    # workaround for https://github.com/saltstack/salt-jenkins/issues/504
                    raise KeyError
                pwd.getpwnam(user)
            except KeyError:
                self.run_call('user.add {0} createhome=False'.format(user))

        # only put userB into the group for the group auth test
        try:
            if salt.utils.platform.is_darwin() and self.group not in str(self.run_call('group.info {0}'.format(self.group))):
                # workaround for https://github.com/saltstack/salt-jenkins/issues/504
                raise KeyError
            grp.getgrnam(self.group)
        except KeyError:
            self.run_call('group.add {0}'.format(self.group))
            self.run_call('user.chgroups {0} {1} True'.format(self.userB, self.group))

    def tearDown(self):
        for user in (self.userA, self.userB):
            try:
                pwd.getpwnam(user)
            except KeyError:
                pass
            else:
                self.run_call('user.delete {0}'.format(user))
        try:
            grp.getgrnam(self.group)
        except KeyError:
            pass
        else:
            self.run_call('group.delete {0}'.format(self.group))

    def test_pam_auth_valid_user(self):
        '''
        test that pam auth mechanism works with a valid user
        '''
        password, hashed_pwd = gen_password()

        # set user password
        set_pw_cmd = "shadow.set_password {0} '{1}'".format(
                self.userA,
                password if salt.utils.platform.is_darwin() else hashed_pwd
        )
        self.run_call(set_pw_cmd)

        # test user auth against pam
        cmd = ('-a pam "*" test.ping '
               '--username {0} --password {1}'.format(self.userA, password))
        resp = self.run_salt(cmd)
        log.debug('resp = %s', resp)
        self.assertTrue(
            'minion:' in resp
        )

    def test_pam_auth_invalid_user(self):
        '''
        test pam auth mechanism errors for an invalid user
        '''
        cmd = ('-a pam "*" test.ping '
               '--username nouser --password {0}'.format('abcd1234'))
        resp = self.run_salt(cmd)
        self.assertTrue(
            'Authentication error occurred.' in ''.join(resp)
        )

    def test_pam_auth_valid_group(self):
        '''
        test that pam auth mechanism works for a valid group
        '''
        password, hashed_pwd = gen_password()

        # set user password
        set_pw_cmd = "shadow.set_password {0} '{1}'".format(
                self.userB,
                password if salt.utils.platform.is_darwin() else hashed_pwd
        )
        self.run_call(set_pw_cmd)

        # test group auth against pam: saltadm is not configured in
        # external_auth, but saltops is and saldadm is a member of saltops
        cmd = ('-a pam "*" test.ping '
               '--username {0} --password {1}'.format(self.userB, password))
        resp = self.run_salt(cmd)
        self.assertTrue(
            'minion:' in resp
        )



# -*- coding: utf-8 -*-

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function
import random

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, MagicMock
from tests.support.unit import TestCase, skipIf

# Import Salt Libs
import salt.modules.cmdmod
import salt.utils.platform
import salt.utils.win_lgpo_auditpol as win_lgpo_auditpol

settings = ['No Auditing', 'Success', 'Failure', 'Success and Failure']


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not salt.utils.platform.is_windows(), 'System is not Windows')
class WinLgpoAuditpolTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {
            win_lgpo_auditpol: {
                '__context__': {},
                '__salt__': {
                    'cmd.run_all': salt.modules.cmdmod.run_all
                }}}

    def test_get_settings(self):
        names = win_lgpo_auditpol._get_valid_names()
        ret = win_lgpo_auditpol.get_settings(category='All')
        for name in names:
            self.assertIn(name, [k.lower() for k in ret])

    def test_get_settings_invalid_category(self):
        self.assertRaises(
            KeyError,
            win_lgpo_auditpol.get_settings,
            category='Fake Category')

    def test_get_setting(self):
        names = win_lgpo_auditpol._get_valid_names()
        for name in names:
            ret = win_lgpo_auditpol.get_setting(name)
            self.assertIn(ret, settings)

    def test_get_setting_invalid_name(self):
        self.assertRaises(
            KeyError,
            win_lgpo_auditpol.get_setting,
            name='Fake Name')

    def test_set_setting(self):
        names = ['Credential Validation', 'IPsec Driver', 'File System', 'SAM']
        mock_set = MagicMock(return_value={'retcode': 0, 'stdout': 'Success'})
        with patch.object(salt.modules.cmdmod, 'run_all', mock_set):
            with patch.object(win_lgpo_auditpol, '_get_valid_names',
                              return_value=[k.lower() for k in names]):
                for name in names:
                    value = random.choice(settings)
                    win_lgpo_auditpol.set_setting(name=name, value=value)
                    switches = win_lgpo_auditpol.settings[value]
                    cmd = 'auditpol /set /subcategory:"{0}" {1}' \
                          ''.format(name, switches)
                    mock_set.assert_called_once_with(cmd=cmd, python_shell=True)
                    mock_set.reset_mock()

    def test_set_setting_invalid_setting(self):
        names = ['Credential Validation', 'IPsec Driver', 'File System']
        with patch.object(win_lgpo_auditpol, '_get_valid_names',
                          return_value=[k.lower() for k in names]):
            self.assertRaises(
                KeyError,
                win_lgpo_auditpol.set_setting,
                name='Fake Name',
                value='No Auditing')

    def test_set_setting_invalid_value(self):
        names = ['Credential Validation', 'IPsec Driver', 'File System']
        with patch.object(win_lgpo_auditpol, '_get_valid_names',
                          return_value=[k.lower() for k in names]):
            self.assertRaises(
                KeyError,
                win_lgpo_auditpol.set_setting,
                name='Credential Validation',
                value='Fake Value')

    def test_get_auditpol_dump(self):
        names = win_lgpo_auditpol._get_valid_names()
        dump = win_lgpo_auditpol.get_auditpol_dump()
        for name in names:
            found = False
            for line in dump:
                if name.lower() in line.lower():
                    found = True
                    break
            self.assertTrue(found)



# -*- coding: utf-8 -*-
'''
Tests for disk state
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

# Import Salt Libs
import salt.states.disk as disk


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DiskTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test disk state
    '''
    def setup_loader_modules(self):
        self.mock_data = {
            '/': {
                '1K-blocks': '41147472',
                'available': '37087976',
                'capacity': '6%',
                'filesystem': '/dev/xvda1',
                'used': '2172880'},
            '/dev': {
                '1K-blocks': '10240',
                'available': '10240',
                'capacity': '0%',
                'filesystem': 'udev',
                'used': '0'},
            '/run': {
                '1K-blocks': '410624',
                'available': '379460',
                'capacity': '8%',
                'filesystem': 'tmpfs',
                'used': '31164'},
            '/sys/fs/cgroup': {
                '1K-blocks': '1026556',
                'available': '1026556',
                'capacity': '0%',
                'filesystem': 'tmpfs',
                'used': '0'}
        }
        self.addCleanup(delattr, self, 'mock_data')
        return {disk: {'__salt__': {'disk.usage': MagicMock(return_value=self.mock_data)}}}

    def test_status_missing(self):
        '''
        Test disk.status when name not found
        '''
        mock_fs = '/mnt/cheese'
        mock_ret = {'name': mock_fs,
                    'result': False,
                    'comment': 'Named disk mount not present ',
                    'changes': {},
                    'data': {}}

        ret = disk.status(mock_fs)
        self.assertEqual(ret, mock_ret)

    def test_status_type_error(self):
        '''
        Test disk.status with incorrectly formatted arguments
        '''
        mock_fs = '/'
        mock_ret = {'name': mock_fs,
                    'result': False,
                    'comment': '',
                    'changes': {},
                    'data': {}}

        mock_ret['comment'] = 'maximum must be an integer '
        ret = disk.status(mock_fs, maximum=r'e^{i\pi}')
        self.assertEqual(ret, mock_ret)

        mock_ret['comment'] = 'minimum must be an integer '
        ret = disk.status(mock_fs, minimum=r'\cos\pi + i\sin\pi')
        self.assertEqual(ret, mock_ret)

    def test_status_range_error(self):
        '''
        Test disk.status with excessive extrema
        '''
        mock_fs = '/'
        mock_ret = {'name': mock_fs,
                    'result': False,
                    'comment': '',
                    'changes': {},
                    'data': {}}

        mock_ret['comment'] = 'maximum must be in the range [0, 100] '
        ret = disk.status(mock_fs, maximum='-1')
        self.assertEqual(ret, mock_ret)

        mock_ret['comment'] = 'minimum must be in the range [0, 100] '
        ret = disk.status(mock_fs, minimum='101')
        self.assertEqual(ret, mock_ret)

    def test_status_inverted_range(self):
        '''
        Test disk.status when minimum > maximum
        '''
        mock_fs = '/'
        mock_ret = {'name': mock_fs,
                    'result': False,
                    'comment': 'minimum must be less than maximum ',
                    'changes': {},
                    'data': {}}

        ret = disk.status(mock_fs, maximum='0', minimum='1')
        self.assertEqual(ret, mock_ret)

    def test_status_threshold(self):
        '''
        Test disk.status when filesystem triggers thresholds
        '''
        mock_min = 100
        mock_max = 0
        mock_fs = '/'
        mock_used = int(self.mock_data[mock_fs]['capacity'].strip('%'))
        mock_ret = {'name': mock_fs,
                    'result': False,
                    'comment': '',
                    'changes': {},
                    'data': self.mock_data[mock_fs]}

        mock_ret['comment'] = 'Disk used space is below minimum of {0} % at {1} %'.format(
                                   mock_min,
                                   mock_used
                               )
        ret = disk.status(mock_fs, minimum=mock_min)
        self.assertEqual(ret, mock_ret)

        mock_ret['comment'] = 'Disk used space is above maximum of {0} % at {1} %'.format(
                                   mock_max,
                                   mock_used
                               )
        ret = disk.status(mock_fs, maximum=mock_max)
        self.assertEqual(ret, mock_ret)

    def test_status_strip(self):
        '''
        Test disk.status appropriately strips unit info from numbers
        '''
        mock_fs = '/'
        mock_ret = {'name': mock_fs,
                    'result': True,
                    'comment': 'Disk used space in acceptable range',
                    'changes': {},
                    'data': self.mock_data[mock_fs]}

        ret = disk.status(mock_fs, minimum='0%')
        self.assertEqual(ret, mock_ret)

        ret = disk.status(mock_fs, minimum='0 %')
        self.assertEqual(ret, mock_ret)

        ret = disk.status(mock_fs, maximum='100%')
        self.assertEqual(ret, mock_ret)

        ret = disk.status(mock_fs, minimum='1024K', absolute=True)
        self.assertEqual(ret, mock_ret)

        ret = disk.status(mock_fs, minimum='1024KB', absolute=True)
        self.assertEqual(ret, mock_ret)

        ret = disk.status(mock_fs, maximum='4194304 KB', absolute=True)
        self.assertEqual(ret, mock_ret)

    def test_status(self):
        '''
        Test disk.status when filesystem meets thresholds
        '''
        mock_min = 0
        mock_max = 100
        mock_fs = '/'
        mock_ret = {'name': mock_fs,
                    'result': True,
                    'comment': 'Disk used space in acceptable range',
                    'changes': {},
                    'data': self.mock_data[mock_fs]}

        ret = disk.status(mock_fs, minimum=mock_min)
        self.assertEqual(ret, mock_ret)

        ret = disk.status(mock_fs, maximum=mock_max)
        self.assertEqual(ret, mock_ret)

        # Reset mock because it's an iterator to run the tests with the
        # absolute flag
        ret = {'name': mock_fs,
               'result': False,
               'comment': '',
               'changes': {},
               'data': {}}

        mock = MagicMock(side_effect=[[], [mock_fs], {mock_fs: {'capacity': '8 %', 'used': '8'}},
            {mock_fs: {'capacity': '22 %', 'used': '22'}},
            {mock_fs: {'capacity': '15 %', 'used': '15'}}])
        with patch.dict(disk.__salt__, {'disk.usage': mock}):
            comt = ('Named disk mount not present ')
            ret.update({'comment': comt})
            self.assertDictEqual(disk.status(mock_fs), ret)

            comt = ('minimum must be less than maximum ')
            ret.update({'comment': comt})
            self.assertDictEqual(disk.status(mock_fs, '10', '20', absolute=True), ret)

            comt = ('Disk used space is below minimum of 10 KB at 8 KB')
            ret.update({'comment': comt, 'data': {'capacity': '8 %', 'used': '8'}})
            self.assertDictEqual(disk.status(mock_fs, '20', '10', absolute=True), ret)

            comt = ('Disk used space is above maximum of 20 KB at 22 KB')
            ret.update({'comment': comt, 'data': {'capacity': '22 %', 'used': '22'}})
            self.assertDictEqual(disk.status(mock_fs, '20', '10', absolute=True), ret)

            comt = ('Disk used space in acceptable range')
            ret.update({'comment': comt, 'result': True,
                'data': {'capacity': '15 %', 'used': '15'}})
            self.assertDictEqual(disk.status(mock_fs, '20', '10', absolute=True), ret)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

# Import Salt Libs
import salt.states.aws_sqs as aws_sqs


@skipIf(NO_MOCK, NO_MOCK_REASON)
class AwsSqsTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.aws_sqs
    '''
    def setup_loader_modules(self):
        return {aws_sqs: {}}

    # 'exists' function tests: 1

    def test_exists(self):
        '''
        Test to ensure the SQS queue exists.
        '''
        name = 'myqueue'
        region = 'eu-west-1'

        ret = {'name': name,
               'result': None,
               'changes': {},
               'comment': ''}

        mock = MagicMock(side_effect=[False, True])
        with patch.dict(aws_sqs.__salt__, {'aws_sqs.queue_exists': mock}):
            comt = 'AWS SQS queue {0} is set to be created'.format(name)
            ret.update({'comment': comt})
            with patch.dict(aws_sqs.__opts__, {'test': True}):
                self.assertDictEqual(aws_sqs.exists(name, region), ret)

            comt = '{0} exists in {1}'.format(name, region)
            ret.update({'comment': comt, 'result': True})
            self.assertDictEqual(aws_sqs.exists(name, region), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        '''
        Test to remove the named SQS queue if it exists.
        '''
        name = 'myqueue'
        region = 'eu-west-1'

        ret = {'name': name,
               'result': None,
               'changes': {},
               'comment': ''}

        mock = MagicMock(side_effect=[True, False])
        with patch.dict(aws_sqs.__salt__, {'aws_sqs.queue_exists': mock}):
            comt = 'AWS SQS queue {0} is set to be removed'.format(name)
            ret.update({'comment': comt})
            with patch.dict(aws_sqs.__opts__, {'test': True}):
                self.assertDictEqual(aws_sqs.absent(name, region), ret)

            comt = '{0} does not exist in {1}'.format(name, region)
            ret.update({'comment': comt, 'result': True})
            self.assertDictEqual(aws_sqs.absent(name, region), ret)
